/**
  ******************************************************************************
  * @file    hw_config.c
  * @author  MCD Application Team
  * @version V2.0.0
  * @date    11-July-2011
  * @brief   This file provides targets hardware configuration.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  */

/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"

/** @addtogroup Embedded_GUI_Example
  * @{
  */

/** @defgroup HWConfig
  * @brief Target Hardware initialization and configuration
  * @{
  */

/** @defgroup HWConfig_Private_Functions
  * @{
  */

/**
  * @brief  Set Touchscreen Controller parameters
  * @param  None
  * @retval None
  */
static void HWConfig_TSCParamInit(void)
{
  TSC_HW_Parameters_TypeDef* pTscParam = NewTscHwParamObj ();

  /* Assign the following values for Touchscreen Controller Parameters Structure */
  pTscParam->TSC_I2C_Clk                = TSC_I2C_CLK;
  pTscParam->TSC_I2C_Sda_Gpio_Pin       = TSC_I2C_SDA_GPIO_PIN;
  pTscParam->TSC_I2C_Sda_Gpio_Port      = TSC_I2C_SDA_GPIO_PORT;
  pTscParam->TSC_I2C_Sda_Gpio_Clk       = TSC_I2C_SDA_GPIO_CLK;
  pTscParam->TSC_I2C_Sda_PinSource      = TSC_I2C_SDA_SOURCE;
  pTscParam->TSC_I2C_Sda_AltFunc        = (uint32_t)TSC_I2C_SDA_AF;
  pTscParam->TSC_I2C_Scl_Gpio_Pin       = TSC_I2C_SCL_GPIO_PIN;
  pTscParam->TSC_I2C_Scl_Gpio_Port      = TSC_I2C_SCL_GPIO_PORT;
  pTscParam->TSC_I2C_Scl_Gpio_Clk       = TSC_I2C_SCL_GPIO_CLK;
  pTscParam->TSC_I2C_Scl_PinSource      = TSC_I2C_SCL_SOURCE;
  pTscParam->TSC_I2C_Scl_AltFunc        = (uint32_t)TSC_I2C_SCL_AF;
  pTscParam->TSC_I2C_Clk                = TSC_I2C_CLK;
  pTscParam->TSC_IT_Exti_Pin_Source     = TSC_IT_EXTI_PIN_SOURCE;
  pTscParam->TSC_IT_Gpio_Clk            = TSC_IT_GPIO_CLK;
  pTscParam->TSC_PortSource             = TSC_GPIO_PORT_SOURCE;
  pTscParam->TSC_PinSource              = TSC_GPIO_PIN_SOURCE;
  pTscParam->TSC_Exti_IrqChannel        = TSC_EXTI_IRQ_CHANNEL;
  pTscParam->TSC_Exti_Line              = TSC_EXTI_LINE;
  pTscParam->TSC_IT_Gpio_Port           = TSC_IT_GPIO_PORT;
  pTscParam->TSC_IT_Gpio_Pin            = TSC_IT_GPIO_PIN;
  pTscParam->TSC_DeviceRegister         = TSC_I2C_DEVICE_REGISTER;
  pTscParam->TSC_Bus_Port               = TSC_I2C_PORT;
}

/**
  * @brief  Set LCD Controller Parameters
  * @param  None
  * @retval None
  */
static void HWConfig_LCDParamInit(void)
{
  LCD_HW_Parameters_TypeDef* pLcdParam = NewLcdHwParamObj ();

  /* Assign the following values for LCD Controller Parameters Structure */
  pLcdParam->LCD_Connection_Mode        = LCD_CONNECTION_MODE;
  pLcdParam->LCD_Rcc_Bus_Periph         = LCD_RCC_BUS_PERIPH;
}

/**
  * @brief  Set evaluation board Parameters parameters
  * @param  None
  * @retval None
  */
void HWConfig_SetHardwareParams(void)
{
  //BTN_HW_Parameters_TypeDef* pBtnParam = NewBtnHwParamObj ();

  HWConfig_TSCParamInit();
  HWConfig_LCDParamInit();

  /* Assign the following values for Button Parameters Structure */
  //pBtnParam->BTN_Rcc_BusPeriph_Gpio      = USER_BUTTON_GPIO_CLK;
  //pBtnParam->BTN_Port                    = USER_BUTTON_GPIO_PORT;
  //pBtnParam->BTN_Pin                     = USER_BUTTON_PIN;
  //pBtnParam->BTN_Pin_Active              = USER_BUTTON_PIN_ACTIVE;
}

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
