/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V2.0.0
  * @date    11-July-2011
  * @brief   This file provides main program functions.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "touchscreen.h"
#include "hw_config.h"
#include "cursor.h"
#include "graphicObject.h"
#include "pictures.h"
#include "uiframework.h"
#include <stdio.h>
#include "stmpe811qtr.h"
#include "stm32f4_discovery_lcd.h"

/** @addtogroup Embedded_GUI_Example
  * @{
  */

/** @defgroup Main
  * @brief Main program body
  * @{
  */

/** @addtogroup Embedded_GUI_Example
  * @{
  */

/** @defgroup Main
  * @brief Main program body
  * @{
  */

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void InputInterface_Init(void);
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  RCC_ClocksTypeDef RCC_Clocks;

  /* Set the Vector Table base location at 0x08000000 */
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0000);
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);

  /* Setup SysTick Timer for 10 msec interrupts  */
  RCC_GetClocksFreq(&RCC_Clocks);
  if (SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 100))
  {
    /* Capture error */
    while (1);
  }
  /* configure Systick priority */
  NVIC_SetPriority(SysTick_IRQn, 0x0B);

  /* Set HW structure parameters */
  HWConfig_SetHardwareParams();

  GL_LCD_Init();         // Inicjalizacja LCD
  GL_Clear(GL_White);    // Czyszczenie ekranu

  GL_SetFont(GL_FONT_SMALL);
  GL_SetTextColor(GL_Black);
  GL_DrawString(300, 100, (uint8_t*)"Press User button to continue...", GL_TRUE);

  STM_EVAL_PBInit(BUTTON_USER, BUTTON_MODE_GPIO);
  while(STM_EVAL_PBGetState(BUTTON_USER) == 0);

  InputInterface_Init(); // Inicjalizacja parametrow zwiazanych z panelem dotykowym

  Show_HomeScreen();     // Utworzenie elementow GUI i wyswietlenie pierwszego okna

  /* Infinite main loop ------------------------------------------------------*/
  while (1)
  {
    ProcessInputData();  // Obsluga dotkniecia panelu
  }
}

/**
  * @brief  Enable input interfaces
  * @param  None
  * @retval None
  */
void InputInterface_Init(void)
{
  TSC_Init();                  // Inicjalizacja
  TS_Calibration();            // Kalibracja
  IOE_ITConfig(IOE_ITSRC_TSC); // Konfiguracja przerwania
}

/**
  * @}
  */

/**
  * @}
  */

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
