/** ****************************************************************************
 *   GxFreeGUI
 *    Module: Drawbox
 *    Author: Jan Szemiet
 *    Version: 0.1
 *  ****************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <stdint.h>
#include <stdlib.h>
#include "Drawbox.h"
#include "graphicObject.h"
#include "LCDHAL.h"

/* Private variables ---------------------------------------------------------*/
static uint16_t CurrentColor;

/* Functions -----------------------------------------------------------------*/
GL_ErrStatus ClearDrawbox (GL_PageControls_TypeDef* pObj)
{
  // Get pointer to GUI object
  GL_Drawbox_TypeDef* pGUIObj = (GL_Drawbox_TypeDef*)(pObj->objPTR);

  uint16_t maxX = pObj->objCoordinates.MaxX;
  uint16_t minY = pObj->objCoordinates.MinY;

  if (!pGUIObj)
  {
    return GL_ERROR;
  }

  GL_SetForegroundColor(GL_Black);

  // Clear Drawbox area
  GL_DrawFilledRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height, GL_White );

  // Draw box
  GL_DrawRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height );

  return GL_OK;
}

void Draw_Point (GL_PageControls_TypeDef* pControl,
                 uint16_t x,
                 uint16_t y)
{
#if 0
  // Draw single pixel
  LCD_PutPixel(x, y, CurrentColor, SinglePixel);
#else
  // Draw little circle
  if( ((pControl->objCoordinates.MaxX)-2 > x) &&
      ((pControl->objCoordinates.MinX)+2 < x) &&
      ((pControl->objCoordinates.MaxY)-2 > y) &&
      ((pControl->objCoordinates.MinY)+2 < y)){
    GL_SetForegroundColor(CurrentColor);
    GL_DrawLine(x+2, y, 1, Vertical);
    GL_DrawLine(x+1, y-1, 3, Vertical);
    GL_DrawLine(x+0, y-2, 5, Vertical);
    GL_DrawLine(x-1, y-1, 3, Vertical);
    GL_DrawLine(x-2, y, 1, Vertical);
  }
#endif
}

GL_PageControls_TypeDef* NewDrawbox (uint16_t ID,
                                     uint16_t Width,
                                     uint16_t Height,
                                     void (*pEventHandler)(uint16_t, uint16_t))
{
  GL_Drawbox_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;

  pGUIObj = (GL_Drawbox_TypeDef*)malloc(sizeof(GL_Drawbox_TypeDef));

  if ( pGUIObj )
  {
    pGUIObj->ID = ID;
    pGUIObj->Width = Width;
    pGUIObj->Height = Height;
    pGUIObj->EventHandler = pEventHandler;

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_DRAWBOX;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

void SetCurrentColor (GL_PageControls_TypeDef* pObj,
                      uint16_t color)
{
  CurrentColor = color;
}

GL_ErrStatus SetDrawboxVisible (GL_PageControls_TypeDef* pObj,
                                GL_Coordinates_TypeDef objCoordinates)
{
  // Get pointer to GUI object
  GL_Drawbox_TypeDef* pGUIObj = (GL_Drawbox_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  if (!pGUIObj)
  {
    return GL_ERROR;
  }

  GL_SetForegroundColor(GL_Black);

  // Clear Drawbox area
  //GL_DrawFilledRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height, GL_White );

  // Draw box
  GL_DrawRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height);

  return GL_OK;
}
