/**
  ******************************************************************************
  * @file    graphicObject.c
  * @author  MCD Application Team
  * @version V2.0.0
  * @date    11-July-2011
  * @brief   This file contains the methods to create the objects that
  *          can be printed on the LCD.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */



#define GL_DrawColorBMP		LCD_DrawColorBMP

/* Includes ------------------------------------------------------------------*/
#include "graphicObject.h"
#include "images.h"
#include "touchscreen.h"
#include "TscHal.h"
//#include "JoyHal.h"
#include "LcdHal.h"
#include "cursor.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Drawbox.h"

/** @addtogroup Embedded_GUI_Library
  * @{
  */

/** @defgroup graphicObject
  * @brief graphicObject functions
  * @{
  */

/* External variables --------------------------------------------------------*/
extern __IO uint16_t GL_TextColor;
extern __IO uint32_t u32_TSXCoordinate;
extern __IO uint32_t u32_TSYCoordinate;

/* External variables --------------------------------------------------------*/
extern void TurnOffTouchInts(void);
extern void TurnOnTouchInts(void);

/* Private typedef -----------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
/** @defgroup graphicObject_Private_Defines
  * @{
  */
#define FONT_LENGTH                     8
#define BUTTON_SLICE_LENGTH             8
#define BUTTON_HEIGHT                   26
#define BUTTON_PIECE_LENGTH             8
#define RBUTTON_OPT_SIZE                20
#define RADIO_BUTTON_RADIUS             9
#define COMBOBOX_SIZE                   22
#define CHECKBOX_SIZE                   20
#define PAGE_MAX_NUM                    50
#define TIMEOUT                         1000000





#define BUTTON_PRESSED_TEXT_COLOR		GL_Yellow
#define BUTTON_NORMAL_TEXT_COLOR		GL_White
#define BUTTON_PRESSED_BG_COLOR       	GL_Grey
#define BUTTON_NORMAL_BG_COLOR          GL_Blue

#define SLIDEBAR_CURSOR_LENGTH          6
#define SLIDEBAR_CENTRAL_LENGTH         27
#define SLIDEBAR_OFFSET_LENGTH          4
#define SLIDEBAR_PIECE_LENGTH           4
#define SLIDEBAR_HEIGHT                 18

#define SWITCH_STATE_OFF_TEXT_COLOR     GL_Red
#define SWITCH_STATE_ON_TEXT_COLOR      GL_Green
#define SWITCH_STATE_OFF_BG_COLOR       GL_Grey
#define SWITCH_STATE_ON_BG_COLOR        GL_Blue


#define SMALL_FONT_WIDTH  8
#define SMALL_FONT_HEIGHT 12


/* !<Enter only even values for this constants>! */
#define SCROLLBAR_ARROW_LENGTH  8
#define SCROLLBAR_THICKNESS     12

/**
  * @}
  */
/* Private macros ------------------------------------------------------------*/
/** @defgroup graphicObject_Private_Macros
  * @{
  */
#define min(x,y)                    ((x<y)? x:y)
#define p_strncpy(oBuf, iBuf, Len)  strncpy((char*)oBuf, (char*)iBuf, (int)Len)
#define p_strlen(iBuf)              strlen((char*)iBuf)
#define p_strcmp(str1,str2)         strcmp((char*)str1,(char*)str2)
#define p_strcpy(str1,str2)         strcpy((char*)str1,(char*)str2)
/**
  * @}
  */

/* Private variables ---------------------------------------------------------*/
/** @defgroup graphicObject_Private_Variables
  * @{
  */
GL_Page_TypeDef* PagesList[PAGE_MAX_NUM];

/*Header and bitmap image of cursor's symbol*/
static uint32_t SlidebarCursorPointer[] =
  {
    0x14, /*Height of cursor symbol*/
    0x06, /*Width of cursor symbol*/
    0x0D, /*Count of pixels of cursor symbol*/
    0x80001, 0x80001, 0x80001, 0x80001, 0x80001, 0x80001, 0x80001
  };

static uint16_t PageCount = 0;
static __IO uint32_t TimingDelay;

__IO uint32_t vu32_gTimeOutCount = 0;
__IO uint8_t vu8_gTouchEnable = 1;
__IO uint8_t vu8_gSleepState = 0;
uint8_t ValidTouchDetected = 0;
/* Set to 0 to disable The Automatic Backlight Switch Off */
__IO uint8_t vu8_gPowerSaveOption = 1;

/* touch_done global variable is used to handle the Touch event/interrupt.
   touch_done must be defined as external variable in main application using this library */
__IO uint8_t touch_done = 0;
__IO uint8_t joy_done = 0;
__IO uint8_t calibration_done = 0;

uint8_t touch_cyclic_check = 0;

/**
  * @}
  */
/* Private function prototypes -----------------------------------------------*/
/** @defgroup graphicObject_Private_FunctionPrototypes
  * @{
  */
static void GL_SetStringFieldValue(uint8_t * dBuf, uint8_t * sBuf, uint32_t MaxLength) ;

static void CallPreEvents(GL_PageControls_TypeDef* pControl);
static void CallEvent(GL_PageControls_TypeDef* pControl);

static GL_ErrStatus Create_Label (GL_Label_TypeDef* );
static GL_ErrStatus Create_Button (GL_Button_TypeDef* );
static GL_ErrStatus Create_RadioButtonGrp (GL_RadioButtonGrp_TypeDef* );
static GL_ErrStatus Create_RadioButtonOption (GL_RadioOption_TypeDef* );
static GL_ErrStatus Create_ComboBoxGrp (GL_ComboBoxGrp_TypeDef* );
static GL_ErrStatus Create_ComboBoxOption (GL_ComboOption_TypeDef* );
static GL_ErrStatus Create_Checkbox (GL_Checkbox_TypeDef* );
static GL_ErrStatus Create_Switch (GL_Switch_TypeDef* );
static GL_ErrStatus Create_Icon (GL_Icon_TypeDef* );
static GL_ErrStatus Create_Slidebar (GL_Slidebar_TypeDef* );
static GL_ErrStatus Create_Histogram (GL_Histogram_TypeDef* );
static GL_ErrStatus Create_GraphChart (GL_GraphChart_TypeDef* );

static GL_ErrStatus SetLabelVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetCheckboxVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetSwitchVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetButtonVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetButtonColorVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetRadioButtonVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetComboBoxVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetIconVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetSlidebarVisible(GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetHistogramVisible( GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );
static GL_ErrStatus SetGraphChartVisible( GL_PageControls_TypeDef* , GL_Coordinates_TypeDef );

static GL_ErrStatus SetListboxVisible (GL_PageControls_TypeDef*, GL_Coordinates_TypeDef);
static GL_ErrStatus SetTextboxVisible (GL_PageControls_TypeDef*, GL_Coordinates_TypeDef);

static GL_ObjDimensions_TypeDef GetObjSize(GL_PageControls_TypeDef* pPageControl);
static GL_Coordinates_TypeDef GetObjCoordinates(GL_Page_TypeDef* pPage, uint16_t ID);
static GL_ErrStatus SetPage(GL_Page_TypeDef* pThis, GL_bool bVal);

/**
  * @}
  */

/* Private functions ---------------------------------------------------------*/
/** @defgroup graphicObject_Private_Functions
  * @{
  */

/**
  * @brief  Displays a circle.
  * @param  Xpos: specifies the X position.
  * @param  Ypos: specifies the Y position.
  * @param  Radius: specifies the circle radius.
  * @param  Color: specifies the filling color.
  * @retval None
  */
void GL_DrawFilledCircle(uint16_t Xpos, uint8_t Ypos, uint16_t Radius, uint16_t Color)
{
  uint32_t n = 2;
  GL_DrawCircle( Xpos - Radius, Ypos - Radius, Radius);
  GL_DrawCircle( Xpos - Radius, Ypos - Radius, Radius - 1);
  GL_SetForegroundColor(Color);
  for (; n < Radius; n++)
  {
    GL_DrawCircle( Xpos - Radius, Ypos - Radius, Radius - n);
  }
}

/**
  * @brief  Displays a line using Bresenham line algorithm.
  * @param  Xpos1: specifies the first point X position
  * @param  Ypos1: specifies the first point Y position
  * @param  Xpos2: specifies the second point X position
  * @param  Ypos2: specifies the second point Y position
  * @param  Color: RGB color of line.
  * @retval None
  */
static void GL_DrawObliqueLine(uint16_t Ypos1, uint16_t Xpos1, uint16_t Ypos2, uint16_t Xpos2, uint16_t Color)
{
  uint8_t steep = 0; /* Steepness of line,0 - non-steep (angle 45-),1 - steep (45+) */
  uint16_t TempPos = 0;
  uint16_t x = 0, y = 0;
  uint16_t deltax = 0, deltay = 0;
  int16_t errorX = 0;
  int8_t ystep = 0;

  /*Case the both points are identical*/
  if ((Xpos1 == Xpos2) && (Ypos1 == Ypos2))
  {
    LCD_PutPixel(Xpos1, Ypos1, Color, SinglePixel);
  }
  /*Case the line are pure horizontal*/
  else if (Ypos1 == Ypos2)
  {
    /*Swap X positions if it is necessary*/
    if (Xpos1 > Xpos2)
    {
      TempPos = Xpos2;
      Xpos2 = Xpos1;
      Xpos1 = TempPos;
    }
    /*Draw first-opening pixel.*/
    LCD_PutPixel(Xpos1, Ypos1, Color, FirstPixel);

    /*Draw middle pixels*/
    for (x = Xpos1 + 1;x < Xpos2;x++)
    {
      LCD_PutPixel(Xpos1, Ypos1, Color, MiddlePixel);
    }

    /*Draw last-ending pixel*/
    LCD_PutPixel(Xpos2, Ypos1, Color, LastPixel);
  }
  /*Case the line are pure vertical*/
  else if (Xpos1 == Xpos2)
  {
    /*Swap Y positions if it is necessary*/
    if (Ypos1 > Ypos2)
    {
      TempPos = Ypos2;
      Ypos2 = Ypos1;
      Ypos1 = TempPos;
    }

    /*Draw line as rectangle of 1 pixel width*/
    //GL_DrawFilledRectangle( Ypos2 - 1, Ypos1 - 1, Xpos2 - 1, Xpos1 - 2, Color);
    GL_DrawFilledRectangle( Xpos2, Ypos1, Xpos2 - Xpos1 - 2, Ypos2 - Ypos1 - 1, Color);
  }
  /*The general line*/
  else
  {
    /*Calculate steepness. If elevation angle is smaller than 45 steep is reset to 0*/
    steep = ((Ypos1 > Ypos2) ? (Ypos1 - Ypos2) : (Ypos2 - Ypos1)) > ((Xpos1 > Xpos2) ? (Xpos1 - Xpos2) : (Xpos2 - Xpos1));

    /*Coordinators X and Y are swapped with each other in case of steep = 1*/
    if (steep != 0x00)
    {
      TempPos = Ypos1;
      Ypos1 = Xpos1;
      Xpos1 = TempPos;

      TempPos = Ypos2;
      Ypos2 = Xpos2;
      Xpos2 = TempPos;
    }
    /*Coordinators X and Y are swapped separetely if it is necessary.*/
    if (Xpos1 > Xpos2)
    {
      TempPos = Ypos1;
      Ypos1 = Ypos2;
      Ypos2 = TempPos;

      TempPos = Xpos2;
      Xpos2 = Xpos1;
      Xpos1 = TempPos;
    }

    /*Bresenham line algorithm */
    deltax = Xpos2 - Xpos1;
    deltay = ((Ypos1 > Ypos2) ? (Ypos1 - Ypos2) : (Ypos2 - Ypos1));
    errorX = -(deltax + 1) / 2;
    y = Ypos1;
    if (Ypos1 < Ypos2)
    {
      ystep = 1;
    }
    else
    {
      ystep = -1;
    }

    for (x = Xpos1; x <= Xpos2 ; x++)
    {
      if (steep != 0x00)
      {
        {
          LCD_PutPixel(y, x, Color, SinglePixel);
        }
      }
      else
      {
        {
          LCD_PutPixel(y, x, Color, SinglePixel);
          //LCD_PutPixel(x, y, Color, SinglePixel);
        }
      }

      errorX = errorX + deltay;
      if (errorX >= 0)
      {
        y = y + ystep;
        errorX = errorX - deltax;
      }
    }
  }
}






/**
  * @brief  Displays a button.
  * @param  maxX: Maximum X coordinate
  * @param  minX: Minimum X coordinate
  * @param  maxY: Maximum Y coordinate
  * @param  minY: Minimum Y coordinate
  * @param  ptrBitmap: pointer to the image
  * @retval None
  */
void GL_DrawButtonBMP(uint16_t maxX, uint16_t minX, uint16_t maxY, uint16_t minY, uint8_t* ptrBitmap)
{
  GL_SetDisplayWindow(maxX, maxY, maxY - minY, maxX - minX);
  GL_DrawBMP(ptrBitmap);
  GL_SetDisplayWindow(LCD_Width - 1, LCD_Height - 1, LCD_Height, LCD_Width);
}

/**
  * @brief  assignes to string fields their values
  * @param  dBuf: destination buffer (string field)
  * @param  sBuf: source buffer (string field  value)
  * @param  MaxLength: The size of the string field
  * @retval None
  */
static void GL_SetStringFieldValue(uint8_t * dBuf, uint8_t * sBuf, uint32_t MaxLength)
{
  uint32_t tmp_length = 0;

  tmp_length = min(p_strlen(sBuf), MaxLength - 1);
  if (tmp_length > 0)
  {
    p_strncpy(dBuf, sBuf, tmp_length);
    dBuf[tmp_length] = 0;
  }
  else
  {
    dBuf[0] = 0;
  }
}











/**
  * @brief  Create and initialize a ComboBox Option object
  * @param  GL_ComboBoxGrp_TypeDef* pThis: ComboBox Group object
  * @param  label: Label for the ComboOption
  * @param  pEventHandler: pointer to function associated to its touch event
  * @retval GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus AddComboOption (GL_ComboBoxGrp_TypeDef* pThis, const char* Label, void (*pEventHandler)(void))
{
  GL_ComboOption_TypeDef *pOptionObj = NULL;
  char* pStr;

  if (!pThis || ( pThis->ComboOptionCount == MAX_COMBO_OPTIONS))
  {
    return GL_ERROR;
  }

  pOptionObj = (GL_ComboOption_TypeDef *)malloc(sizeof(GL_ComboOption_TypeDef));

  if (pOptionObj)
  {
    pOptionObj->ComboBoxGrp = pThis;

    pStr = malloc(strlen((char*)Label));
    if ( strlen((char*)Label) > MAX_COMBO_LABEL_LENGTH )
    {
      // Only the first MAX_COMBO_LABEL_LENGTH characters of the slidebar label can be printed
      strncpy(pStr,(char*)Label, MAX_COMBO_LABEL_LENGTH);
      pStr[MAX_COMBO_LABEL_LENGTH] = '\0';   // Null character manually added
    }
    else
    {
      strcpy(pStr,(char*)Label);
    }

    pOptionObj->Label = (uint8_t*)pStr;
    pOptionObj->EventHandler = pEventHandler;

    // Create the ComboBox Option object
    Create_ComboBoxOption(pOptionObj);



    pThis->ComboOptions[pThis->ComboOptionCount] = pOptionObj;
    pThis->ComboOptionCount++;

    if (pThis->ComboOptionCount == 1)
    {
      ((GL_ComboOption_TypeDef*)(pThis->ComboOptions[0]))->Active = GL_TRUE;
    }
  }

  return GL_OK;
}



/**
  * @brief  Create and initialize a Icon object
  * @param  oName: Object Name
  * @param  Image_PTR: Icon image pointer
  * @param  Width: Image Width
  * @param  Height: Image Height
  * @param  pEventHandler: pointer to function associated to its touch event
  * @retval GL_PageControls_TypeDef* - The created Object pointer
  */
GL_PageControls_TypeDef* NewIcon (uint16_t ID, const uint8_t* Image_PTR, uint16_t Width, uint8_t Height, void (*pEventHandler)(void))
{
  GL_Icon_TypeDef *pControlObj = NULL;
  GL_PageControls_TypeDef* pPageControlObj = NULL;

  pControlObj = (GL_Icon_TypeDef *)malloc(sizeof(GL_Icon_TypeDef));
  if (pControlObj)
  {
    pControlObj->ID = ID;
    pControlObj->ImagePTR = (uint8_t*)Image_PTR;
    pControlObj->ImageWidth = Width;
    pControlObj->ImageHeight = Height;
    pControlObj->EventHandler = pEventHandler;

    /* Create the Icon object */
    Create_Icon(pControlObj);

    pPageControlObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));
    if ( pPageControlObj )
    {
      pPageControlObj->objPTR = (void*)pControlObj;
      pPageControlObj->objType = GL_ICON;
    }
    else
    {
      free(pControlObj);
      pControlObj = NULL;
    }
  }
  return pPageControlObj;
}










/**
  * @brief  Create and initialize a Histogram object
  * @param  oName: Object Name
  * @param  labelX: Label of the X axis
  * @param  labelY: Label of the Y axis
  * @param  data_points[]: The array of points to be plot on the LCD
  * @param  n_points: Number of points to be plot
  * @retval GL_PageControls_TypeDef* - The created Object pointer
  */
GL_PageControls_TypeDef* NewHistogram ( uint16_t ID, const char* LabelX, const char* LabelY, int16_t data_points[], uint8_t n_points )
{
  GL_PageControls_TypeDef *pPageObj = NULL;
  GL_Histogram_TypeDef *pGUIObj = (GL_Histogram_TypeDef *)malloc(sizeof(GL_Histogram_TypeDef));

  if (pGUIObj)
  {
    pGUIObj->ID = ID;
    pGUIObj->Label_X = (uint8_t*)LabelX;
    pGUIObj->Label_Y = (uint8_t*)LabelY;
    pGUIObj->NrOfPoints = 0;

    //GL_SetStringFieldValue(, (uint8_t *)labelX, MAX_HIST_LABEL_LENGTH);
    //GL_SetStringFieldValue(pControlObj->Label_Y, (uint8_t *)labelY, MAX_HIST_LABEL_LENGTH);

    /* Determine the length of the array of points and copy the content into the
       internal points array limiting the number of points to the maximum allowed */
    if ( n_points > MAX_HIST_POINTS )
    {
      memcpy(pGUIObj->Points, data_points, MAX_HIST_POINTS * sizeof(int16_t));
      pGUIObj->NrOfPoints = MAX_HIST_POINTS;
    }
    else
    {
      memcpy(pGUIObj->Points, data_points, n_points * sizeof(int16_t));
      pGUIObj->NrOfPoints =  n_points;
    }

    /* Create the Histogram object */
    Create_Histogram(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));
    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_HISTOGRAM;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Create and initialize a Graphical Chart object
  * @param  oName: Object Name
  * @param  labelX: Label of the X axis
  * @param  labelY: Label of the Y axis
  * @param  data_points[]: The array of points to be plot on the LCD
  * @param  n_points: Number of points to be plot
  * @param  Background: Select if the black background has to be used
  *         This parameter can be one of the following values:
  *     @arg  GL_TRUE
  *     @arg  GL_FALSE
  * @retval GL_PageControls_TypeDef* - The created Object pointer
  */
GL_PageControls_TypeDef* NewGraphChart (uint16_t ID,
                                        const char* LabelX,
                                        const char* LabelY,
                                        int16_t data_points[],
                                        uint8_t n_points,
                                        GL_bool Background)
{
  GL_PageControls_TypeDef *pPageObj = NULL;
  GL_GraphChart_TypeDef *pGUIObj = (GL_GraphChart_TypeDef *)malloc(sizeof(GL_GraphChart_TypeDef));

  if (pGUIObj)
  {
    pGUIObj->ID = ID;
    pGUIObj->Label_X = (uint8_t*)LabelX;
    pGUIObj->Label_Y = (uint8_t*)LabelY;
    pGUIObj->NrOfPoints = 0;

    //GL_SetStringFieldValue(pControlObj->label_X, (uint8_t *)LabelX, MAX_GRAPH_LABEL_LENGTH);
    //GL_SetStringFieldValue(pControlObj->label_Y, (uint8_t *)LabelY, MAX_GRAPH_LABEL_LENGTH);

    /* Determine the length of the array of points and copy the content into the
       internal points array limiting the number of points to the maximum allowed */
    if ( n_points > MAX_GRAPH_POINTS )
    {
      memcpy(pGUIObj->Points, data_points, MAX_GRAPH_POINTS*2);
      pGUIObj->NrOfPoints = MAX_GRAPH_POINTS;
    }
    else
    {
      memcpy(pGUIObj->Points, data_points, n_points*2);
      pGUIObj->NrOfPoints = n_points;
    }

    pGUIObj->Background = Background;

    /* Create the Graphical Chart object */
    Create_GraphChart(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));
    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_GRAPH_CHART;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Add a new Control object to the page
  * @param  PosX: Coordinate for X axis
  * @param  PosY: Coordinate for Y axis
  * @param  *objPTR: Pointer to Object Structure
  * @param  *pagePTR: Pointer to Page Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus AddPageControlObj (uint16_t PosX,
                                uint16_t PosY,
                                GL_PageControls_TypeDef* objPTR,
                                GL_Page_TypeDef* pagePTR)
{
  if (pagePTR->ControlCount < MAX_CTRL_X_PAGE)
  {
    GL_Coordinates_TypeDef objCoordinates;
    GL_ComboBoxGrp_TypeDef* pTmpComboBoxGrp = NULL;
    GL_ComboBoxGrp_TypeDef* pTmpComboBoxGrp1 = NULL;

    memset(&objCoordinates, 0x00, sizeof(GL_Coordinates_TypeDef));
    if ((PosX > LCD_Width-1) || (PosY > LCD_Height-1))
    {
      return GL_ERROR;
    }

    objCoordinates.MaxX = PosX;
    objCoordinates.MinY = PosY;



    if (objPTR->objType == GL_COMBOBOX)
    {
      GL_PageControls_TypeDef* tmpComboBoxGrpBrother;

      objCoordinates.MinX = PosX - COMBOBOX_SIZE + 1;
      objCoordinates.MaxY = PosY + COMBOBOX_SIZE - 1;

      objPTR->SetObjVisible = SetComboBoxVisible;
      objPTR->objCoordinates = objCoordinates;
      pagePTR->PageControls[pagePTR->ControlCount] = objPTR;
      pagePTR->ControlCount++;
      pTmpComboBoxGrp1 = ((GL_ComboBoxGrp_TypeDef*)(objPTR->objPTR));
      objPTR->ID = pTmpComboBoxGrp1->ID;

      if ( pTmpComboBoxGrp1->Secondary == GL_FALSE)
      {
        tmpComboBoxGrpBrother = NewComboBoxGrp(00);
        pTmpComboBoxGrp = ((GL_ComboBoxGrp_TypeDef*)(tmpComboBoxGrpBrother->objPTR));
        pTmpComboBoxGrp->PrimaryComboOpt = objPTR;
        pTmpComboBoxGrp->Secondary = GL_TRUE;
#ifndef USE_2D_OBJECTS
        pTmpComboBoxGrp->ImageUnClickedPTR = (uint8_t*)ArrowDownUntouched;
        pTmpComboBoxGrp->ImageClickedPTR = (uint8_t*)ArrowDownTouched;
#endif
        AddPageControlObj( PosX - COMBOBOX_SIZE,
                           PosY,
                           tmpComboBoxGrpBrother,
                           pagePTR );
      }
      return GL_OK;
    }

    // Jezeli obiektem GUI jest przycisk lub przelacznik
    else if (objPTR->objType == GL_BUTTON || objPTR->objType == GL_SWITCH)
    {
      if(objPTR->objType == GL_BUTTON)
      {
        GL_Button_TypeDef* pTmp = ((GL_Button_TypeDef*)(objPTR->objPTR));  // Pobierz wskaznik na obiekt graficzny
        objPTR->SetObjVisible = SetButtonVisible;                          // Przypisz funkcje rysujaca dany obiekt graficzny
        objPTR->ID = pTmp->ID;                                             // Skopiuj ID do struktury nadrzednej
      }
      else
      {
        GL_Switch_TypeDef* pTmp = ((GL_Switch_TypeDef*)(objPTR->objPTR));
        objPTR->SetObjVisible = SetSwitchVisible;
        objPTR->ID = pTmp->ID;
      }

      objCoordinates.MinX = PosX - BUTTON_PIECE_LENGTH;                    // Przypisz wstepne wymiary obiektu
      objCoordinates.MaxY = PosY + BUTTON_HEIGHT;
    }
    else if (objPTR->objType == GL_BUTTON_COLOR)
    {
      GL_ButtonColor_TypeDef* pTmp = ((GL_ButtonColor_TypeDef*)(objPTR->objPTR));  // Pobierz wskaznik na obiekt graficzny
      objPTR->SetObjVisible = SetButtonColorVisible;                       // Przypisz funkcje rysujaca dany obiekt graficzny
      objPTR->ID = pTmp->ID;                                               // Skopiuj ID do struktury nadrzednej

      objCoordinates.MinX = PosX - 28;                                     // Przypisz wstepne wymiary obiektu
      objCoordinates.MaxY = PosY + 28;
    }
    else if (objPTR->objType == GL_CHECKBOX)
    {
      GL_Checkbox_TypeDef* pTmp = ((GL_Checkbox_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetCheckboxVisible;
      objPTR->ID = pTmp->ID;

      objCoordinates.MinX = PosX - CHECKBOX_SIZE;
      objCoordinates.MaxY = PosY + CHECKBOX_SIZE;
    }

    else if(objPTR->objType == GL_GRAPH_CHART)
    {
      GL_GraphChart_TypeDef* pTmp = ((GL_GraphChart_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetGraphChartVisible;
      objPTR->ID = pTmp->ID;
    }
    else if (objPTR->objType == GL_HISTOGRAM)
    {
      GL_Histogram_TypeDef* pTmp = ((GL_Histogram_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetHistogramVisible;
      objPTR->ID = pTmp->ID;
    }
    else if (objPTR->objType == GL_LABEL)
    {
      GL_Label_TypeDef* pTmp = ((GL_Label_TypeDef*)(objPTR->objPTR));  // Pobierz wskaznik na obiekt graficzny
      objPTR->SetObjVisible = SetLabelVisible;                         // Przypisz funkcje rysujaca dany obiekt graficzny
      objPTR->ID = pTmp->ID;                                           // Skopiuj ID do struktury nadrzednej
    }
    else if (objPTR->objType == GL_LISTBOX)
    {
      GL_Listbox_TypeDef* pTmp = ((GL_Listbox_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetListboxVisible;
      objPTR->ID = pTmp->ID;

      objCoordinates.MinX = PosX - pTmp->Width + 1;
      objCoordinates.MaxY = PosY + pTmp->Height - 1;
    }
    else if (objPTR->objType == GL_RADIO_BUTTON)
    {
      GL_RadioOption_TypeDef* pTmp = ((GL_RadioOption_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetRadioButtonVisible;
      objPTR->ID = pTmp->RadioButtonGrp->ID;//?

      objCoordinates.MinX = PosX - RBUTTON_OPT_SIZE + 1;
      objCoordinates.MaxY = PosY + RBUTTON_OPT_SIZE - 1;
    }
    else if (objPTR->objType == GL_SLIDEBAR)
    {
      GL_Slidebar_TypeDef* pTmp = ((GL_Slidebar_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetSlidebarVisible;
      objPTR->ID = pTmp->ID;

      if (pTmp->Direction == GL_HORIZONTAL)
      {
        objCoordinates.MinX = PosX - SLIDEBAR_PIECE_LENGTH + 1;
        objCoordinates.MaxY = PosY + SLIDEBAR_HEIGHT - 1;
      }
      else if (pTmp->Direction == GL_LEFT_VERTICAL)
      {
        objCoordinates.MinX = PosX - SLIDEBAR_HEIGHT + 1;
        objCoordinates.MaxY = PosY + (SLIDEBAR_CENTRAL_LENGTH + 2) * SLIDEBAR_PIECE_LENGTH - 1;
      }
      else if (pTmp->Direction == GL_RIGHT_VERTICAL)
      {
        objCoordinates.MaxX = PosX - SMALL_FONT_HEIGHT- 1;
        objCoordinates.MinX = PosX - SLIDEBAR_HEIGHT + 1 - GL_FontHeight - 1;
        objCoordinates.MaxY = PosY + (SLIDEBAR_CENTRAL_LENGTH + 2) * SLIDEBAR_PIECE_LENGTH + SLIDEBAR_PIECE_LENGTH - 1;
      }
    }
    else if (objPTR->objType == GL_ICON)
    {
      GL_Icon_TypeDef* pTmp = ((GL_Icon_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetIconVisible;
      objPTR->ID = pTmp->ID;

      objCoordinates.MinX = PosX - pTmp->ImageWidth;
      objCoordinates.MaxY = PosY + pTmp->ImageHeight;
    }
    else if (objPTR->objType == GL_TEXTBOX)
    {
      GL_Textbox_TypeDef* pTmp = ((GL_Textbox_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetTextboxVisible;
      objPTR->ID = pTmp->ID;

      objCoordinates.MinX = PosX - pTmp->Width + 1;
      objCoordinates.MaxY = PosY + pTmp->Height - 1;
    }
    else if (objPTR->objType == GL_DRAWBOX)
    {
      GL_Drawbox_TypeDef* pTmp = ((GL_Drawbox_TypeDef*)(objPTR->objPTR));
      objPTR->SetObjVisible = SetDrawboxVisible;
      objPTR->ID = pTmp->ID;

      objCoordinates.MinX = PosX - pTmp->Width + 1;
      objCoordinates.MaxY = PosY + pTmp->Height - 1;
    }

    objPTR->objCoordinates = objCoordinates;

    pagePTR->PageControls[pagePTR->ControlCount] = objPTR;
    pagePTR->ControlCount++;
    return GL_OK;
  }
  else
    return GL_ERROR;
}

/**
  * @brief  Return the label of a Label object
  * @param  *pPage: Pointer to Page Structure
  * @param  ID: Object Identifier
  * @param  label: Variable where to copy the label of a Label Object
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus Get_Label ( GL_Page_TypeDef* pPage, uint16_t ID, uint8_t* label )
{
  GL_Label_TypeDef* pTmp;
  uint32_t index = 0;

  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required label */
  {
    if (pPage->PageControls[index]->objType == GL_LABEL)
    {
      pTmp = (GL_Label_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID)
      {
        GL_SetStringFieldValue(label, (uint8_t *)pTmp->Label, MAX_LABEL_LENGTH);
        return GL_OK;
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  Return the current value of cursor position in a slidebar object
  * @param  *pPage: Pointer to Page Structure
  * @param  ID: Object Identifier
  * @retval uint8_t - The Cursor Position value (0-100 percentage)
  */
uint8_t Get_SlidebarValue ( GL_Page_TypeDef* pPage, uint16_t ID )
{
  GL_Slidebar_TypeDef* pTmp = NULL;
  uint32_t index = 0;

  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required label */
  {
    if (pPage->PageControls[index]->objType == GL_SLIDEBAR)
    {
      pTmp = (GL_Slidebar_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pTmp->CurrentValue;
      }
    }
    index++;
  }
  return GL_NULL;
}

/**
  * @brief  Assign Label Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Label (GL_Label_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  return GL_OK;
}

/**
  * @brief  Assign Button Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Button (GL_Button_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  pThis->ObjectTouched = GL_FALSE;
  return GL_OK;
}

static GL_ErrStatus Create_ButtonColor (GL_ButtonColor_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  pThis->ObjectTouched = GL_FALSE;
  return GL_OK;
}

/**
  * @brief  Assign RadioButtonGrp Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_RadioButtonGrp (GL_RadioButtonGrp_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->RadioOptionCount = 0;
  /* Assign member functions */
  pThis->AddRadioOption = AddRadioOption;
  return GL_OK;
}

/**
  * @brief  Assign RadioButton Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_RadioButtonOption (GL_RadioOption_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  pThis->Checked = GL_FALSE;

  return GL_OK;
}

/**
  * @brief  Assign ComboBoxGrp Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_ComboBoxGrp (GL_ComboBoxGrp_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
#ifndef USE_2D_OBJECTS
  pThis->ImageUnClickedPTR = (uint8_t*)ArrowUpUntouched;
  pThis->ImageClickedPTR = (uint8_t*)ArrowUpTouched;
#endif
  pThis->Control_Visible = GL_TRUE;
  pThis->Secondary = GL_FALSE;
  pThis->ComboOptionCount = 0;
  /* Assign member functions */
  pThis->AddComboOption = AddComboOption;

  return GL_OK;
}

/**
  * @brief  Assign ComboBox Option Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_ComboBoxOption (GL_ComboOption_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->BackColor = GL_White;
  pThis->TextColor = GL_Blue;
  pThis->Active = GL_FALSE;

  return GL_OK;
}

/**
  * @brief  Assign Checkbox Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Checkbox (GL_Checkbox_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Checked = GL_FALSE;
  pThis->Control_Visible = GL_TRUE;
  return GL_OK;
}

/**
  * @brief  Assign Slidebar Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Slidebar (GL_Slidebar_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  /*Allocate memory for cursor behind array. Actual behind of cursor is stored here*/
  pThis->BehindCursor = (uint16_t *)malloc(SlidebarCursorPointer[2] * sizeof(uint16_t));
  return GL_OK;
}

/**
  * @brief  Assign Switch Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Switch (GL_Switch_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  pThis->ObjectTouched = GL_FALSE;
  return GL_OK;
}

/**
  * @brief  Assign Icon Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Icon (GL_Icon_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->ObjectTouched = GL_FALSE;
  pThis->Control_Visible = GL_TRUE;
  return GL_OK;
}

/**
  * @brief  Assign Histogram Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_Histogram (GL_Histogram_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  return GL_OK;
}

/**
  * @brief  Assign GraphChart Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus Create_GraphChart (GL_GraphChart_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  return GL_OK;
}

/**
  * @brief  Assign PageObj Methods/Properties members.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus Create_PageObj (GL_Page_TypeDef* pThis)
{
  if (!pThis)
  {
    return GL_ERROR;
  }
  else
  {
    pThis->Page_Active       = GL_FALSE;
    pThis->Page_Visible      = GL_FALSE;
    /* Assign member functions */
    pThis->ShowPage          = ShowPage;
    pThis->SetPage           = SetPage;
    pThis->GetObjStatus      = GetObjStatus;
    pThis->GetObjCoordinates = GetObjCoordinates;
    pThis->ControlCount      = 0;

    memset(pThis->PageControls,0x00,sizeof(GL_PageControls_TypeDef*)*MAX_CTRL_X_PAGE);

    /* pThis->GetObjSize = GetObjSize; */
    pThis->SetPage(pThis, GL_FALSE);
    if ( PageCount < PAGE_MAX_NUM )
    {
      PagesList[PageCount] = pThis;
      PageCount++;
    }
    else
    {
      return GL_ERROR;
    }
  }
  return GL_OK;
}

/**
  * @brief  Free the allocated memory for a PageControl object
  * @param  *pPageControlObj: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus DestroyPageControl ( GL_Page_TypeDef* pPage, uint16_t ID )
{
  uint32_t index = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }

  /* search for the required object */
  while ( index < pPage->ControlCount )
  {
    if (pPage->PageControls[index]->objType == GL_LABEL)
    {
      GL_Label_TypeDef* pTmp;
      pTmp = (GL_Label_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_BUTTON)
    {
      GL_Button_TypeDef* pTmp;
      pTmp = (GL_Button_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_SWITCH)
    {
      GL_Switch_TypeDef* pTmp;
      pTmp = (GL_Switch_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_CHECKBOX)
    {
      GL_Checkbox_TypeDef* pTmp;
      pTmp = (GL_Checkbox_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_RADIO_BUTTON)
    {
      GL_RadioButtonGrp_TypeDef* pTmp;
      pTmp = (GL_RadioButtonGrp_TypeDef*)(((GL_RadioOption_TypeDef*)pPage->PageControls[index]->objPTR)->RadioButtonGrp);

      if (pTmp->ID == ID )
      {
        if (--pTmp->RadioOptionCount == 0)
        {
          free(pTmp);
        }
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      uint8_t free_counter = 0;
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        for ( free_counter = 0; free_counter < pTmp->ComboOptionCount; free_counter++)
        {
          free(pTmp->ComboOptions[free_counter]);
        }
        pTmp->ComboOptionCount = 0;
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_ICON)
    {
      GL_Icon_TypeDef* pTmp;
      pTmp = (GL_Icon_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_SLIDEBAR)
    {
      GL_Slidebar_TypeDef* pTmp;
      pTmp = (GL_Slidebar_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        if (pTmp->BehindCursor != NULL)
        {
          free(pTmp->BehindCursor);
          pTmp->BehindCursor = NULL;
        }
        memset(pPage->PageControls[index]->objPTR, 0x00, sizeof(GL_Slidebar_TypeDef));
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_HISTOGRAM)
    {
      GL_Histogram_TypeDef* pTmp;
      pTmp = (GL_Histogram_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    else if (pPage->PageControls[index]->objType == GL_GRAPH_CHART)
    {
      GL_GraphChart_TypeDef* pTmp;
      pTmp = (GL_GraphChart_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        free(pPage->PageControls[index]->objPTR);
        pPage->PageControls[index]->objPTR = GL_NULL;
        free(pPage->PageControls[index]);
        if (index != pPage->ControlCount - 1)
        {
          pPage->PageControls[index] = pPage->PageControls[pPage->ControlCount-1];
          pPage->PageControls[pPage->ControlCount-1] = GL_NULL;
          pPage->ControlCount--;
        }
        else
        {
          pPage->PageControls[index] = GL_NULL;
          pPage->ControlCount--;
        }
        return GL_OK;
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  Free the allocated memory for all the PageControl objects belonging to a Page
  * @param  *pThis: Pointer to Page Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus DestroyPage (GL_Page_TypeDef * pPage)
{
  int32_t index = 0;

  if (!pPage)
  {
    return GL_ERROR;
  }

  index = pPage->ControlCount - 1;
  while ( index >= 0 ) /* destroy all the PageControl object of the given Page */
  {
    if (pPage->PageControls[index]->objType == GL_LABEL)
    {
      GL_Label_TypeDef* pTmp;
      pTmp = (GL_Label_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_BUTTON)
    {
      GL_Button_TypeDef* pTmp;
      pTmp = (GL_Button_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_SWITCH)
    {
      GL_Switch_TypeDef* pTmp;
      pTmp = (GL_Switch_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_CHECKBOX)
    {
      GL_Checkbox_TypeDef* pTmp;
      pTmp = (GL_Checkbox_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_RADIO_BUTTON)
    {
      GL_RadioButtonGrp_TypeDef* pTmp;
      pTmp = (GL_RadioButtonGrp_TypeDef*)(((GL_RadioOption_TypeDef*)pPage->PageControls[index]->objPTR)->RadioButtonGrp);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_ICON)
    {
      GL_Icon_TypeDef* pTmp;
      pTmp = (GL_Icon_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_SLIDEBAR)
    {
      GL_Slidebar_TypeDef* pTmp;
      pTmp = (GL_Slidebar_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_HISTOGRAM)
    {
      GL_Histogram_TypeDef* pTmp;
      pTmp = (GL_Histogram_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    else if (pPage->PageControls[index]->objType == GL_GRAPH_CHART)
    {
      GL_GraphChart_TypeDef* pTmp;
      pTmp = (GL_GraphChart_TypeDef*)(pPage->PageControls[index]->objPTR);
      DestroyPageControl( pPage, pTmp->ID );
    }
    index--;
  }
  pPage->ControlCount = 0;
  return GL_OK;
}


/**
  * @brief  Displays a Cross.
  * @param  Xpos: specifies the left X position.
  * @param  Ypos: specifies the botton Y position.
  * @retval None
  */
void GL_Cross(uint16_t Ypos, uint16_t Xpos)
{
  GL_DrawLine(Xpos + 30, Ypos, 60, GL_Horizontal); /* Horizontal Line */
  GL_DrawLine(Xpos, Ypos - 30, 60, GL_Vertical);   /* Vertical Line   */
}

































/**
  * @brief  Creates and initializes a RadioButton Option object
  * @param  GL_RadioButtonGrp_TypeDef* pThis: RadioButton Group object
  * @param  label: Label for the RadioButton Option
  * @param  pEventHandler: pointer to function associated to its touch event
  * @retval GL_PageControls_TypeDef* - The created Object pointer
  */
GL_PageControls_TypeDef* AddRadioOption (GL_RadioButtonGrp_TypeDef* pObj,
                                         const char* Label,
                                         void (*pEventHandler)(void))
{
  GL_RadioOption_TypeDef *pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr = NULL;

  if (!pObj || pObj->RadioOptionCount == 3)
  {
    return GL_ERROR;
  }

  pGUIObj = (GL_RadioOption_TypeDef *)malloc(sizeof(GL_RadioOption_TypeDef));

  if (pGUIObj)
  {
    pStr = malloc(strlen((char*)Label));
    if ( strlen((char*)Label) > MAX_RADIO_OPTION_LABEL_LENGTH )
    {
      // Only the first MAX_RADIO_OPTION_LABEL_LENGTH characters of the radio option label can be printed
      strncpy(pStr,(char*)Label, MAX_RADIO_OPTION_LABEL_LENGTH);
      pStr[MAX_RADIO_OPTION_LABEL_LENGTH] = '\0';   // Null character manually added
    }
    else
    {
      strcpy(pStr,(char*)Label);
    }

    pGUIObj->RadioButtonGrp = pObj;
    pGUIObj->Label = (uint8_t*)pStr;
    pGUIObj->EventHandler = pEventHandler;

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_RADIO_BUTTON;

      // Create the RadioButton Option object
      Create_RadioButtonOption(pGUIObj);

      pObj->RadioOptions[pObj->RadioOptionCount] = pPageObj;
      pObj->RadioOptionCount++;
      if (pObj->RadioOptionCount == 1)
      {
        ((GL_RadioOption_TypeDef*)(pObj->RadioOptions[0]->objPTR))->Checked = GL_TRUE;
      }
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a Button object
  * @param  ID:            object ID.
  * @param  Label:         text on a Button Object.
  * @param  pEventHandler: pointer to function associated to its touch event.
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewButton (uint16_t ID,
                                    const char* Label,
                                    void (*pEventHandler)(void))
{
  GL_Button_TypeDef* pGUIObj = (GL_Button_TypeDef*)malloc(sizeof(GL_Button_TypeDef));
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr = NULL;

  if ( pGUIObj )
  {
    pStr = malloc(strlen((char*)Label));
    strcpy(pStr,(char*)Label);

    pGUIObj->ID = ID;
    pGUIObj->Label = (uint8_t*)pStr;
    pGUIObj->EventHandler = pEventHandler;

    // Create the Button object
    Create_Button(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_BUTTON;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

GL_PageControls_TypeDef* NewButtonColor (uint16_t ID,
                                         uint16_t color,
                                         void (*pEventHandler)(void))
{
GL_ButtonColor_TypeDef* pGUIObj = (GL_ButtonColor_TypeDef*)malloc(sizeof(GL_ButtonColor_TypeDef));
GL_PageControls_TypeDef* pPageObj = NULL;

if ( pGUIObj )
{
  pGUIObj->ID = ID;
  pGUIObj->Color = color;
  pGUIObj->EventHandler = pEventHandler;

  // Create the ButtonColor object
  Create_ButtonColor(pGUIObj);

  pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

  if ( pPageObj )
  {
    pPageObj->objPTR = (void*)pGUIObj;
    pPageObj->objType = GL_BUTTON_COLOR;
  }
  else
  {
    free(pGUIObj);
    pGUIObj = NULL;
  }
}
return pPageObj;
}

/**
  * @brief  Creates and initializes a Checkbox object
  * @param  ID:            object ID.
  * @param  Label:         text for the Checkbox.
  * @param  pEventHandler: pointer to function associated to its touch event.
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewCheckbox (uint16_t ID,
                                      const char* Label,
                                      void (*pEventHandler)(void))
{
  GL_Checkbox_TypeDef *pGUIObj = (GL_Checkbox_TypeDef *)malloc(sizeof(GL_Checkbox_TypeDef));
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr = NULL;

  if ( pGUIObj )
  {
    pStr = malloc(strlen((char*)Label));
    strcpy(pStr,(char*)Label);

    pGUIObj->ID = ID;
    pGUIObj->Label = (uint8_t*)pStr;
    pGUIObj->EventHandler = pEventHandler;

    // Create the Checkbox object
    Create_Checkbox(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_CHECKBOX;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a ComboBox Group object
  * @param  ID:       object ID.
  * @retval GL_PageControls_TypeDef* - the created Object pointer
  */
GL_PageControls_TypeDef* NewComboBoxGrp (uint16_t ID)
{
  GL_ComboBoxGrp_TypeDef* pComboBoxGrp = (GL_ComboBoxGrp_TypeDef *)malloc(sizeof(GL_ComboBoxGrp_TypeDef));
  GL_PageControls_TypeDef* pPageObj = NULL;

  if (pComboBoxGrp)
  {
    pComboBoxGrp->ID = ID;

    // Create the ComboBox object
    Create_ComboBoxGrp(pComboBoxGrp);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pComboBoxGrp;
      pPageObj->objType = GL_COMBOBOX;
    }
    else
    {
      free(pComboBoxGrp);
      pComboBoxGrp = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a Label object
  * @param  ID:       object ID.
  * @param  Label:    text on a Label Object.
  * @param  Direction: the printing orientation.
  *         This parameter can be one of the following values:
  *     @arg  GL_Vertical
  *     @arg  GL_Horizontal
  * @param  FontSize: the size of the font.
  *         This parameter can be one of the following values:
  *     @arg  GL_FONT_BIG
  *     @arg  GL_FONT_SMALL
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewLabel (uint16_t ID,
                                   const char* Label,
                                   GL_Direction Direction,
                                   uint8_t FontSize,
                                   uint16_t Colour)
{
  GL_Label_TypeDef* pGUIObj = (GL_Label_TypeDef*)malloc(sizeof(GL_Label_TypeDef));
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr = NULL;

  if ( pGUIObj )
  {
    pStr = malloc(strlen((char*)Label));
    strcpy(pStr,(char*)Label);

    pGUIObj->ID = ID;
    pGUIObj->Label = (uint8_t*)pStr;
    pGUIObj->Direction = Direction;
    pGUIObj->FontSize = FontSize;
    pGUIObj->Colour = Colour;

    // Create the Label object
    Create_Label(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_LABEL;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a listbox object
  * @param  ID:             object ID.
  * @param  Width:          listbox width.
  * @param  Height:         listbox height.
  * @param  MultiSelection:
  *         GL_FALSE - only one item can be selected at the same time
  *         GL_TRUE - two or more items can be selected at the same time
  * @param  *pEventHandler: pointer to function associated to its touch event.
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewListbox (uint16_t ID,
                                     uint16_t Width,
                                     uint16_t Height,
                                     GL_bool MultiSelection,
                                     void (*pEventHandler)(uint16_t))
{
  GL_Listbox_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;

  pGUIObj = (GL_Listbox_TypeDef*)malloc(sizeof(GL_Listbox_TypeDef));

  if ( pGUIObj )
  {
    pGUIObj->ID = ID;
    pGUIObj->Width = Width;
    pGUIObj->Height = Height;
    pGUIObj->NrOfItems = 0;
    pGUIObj->NrOfFirstItem = 0;
    pGUIObj->NrOfLastSelItem = 0;
    pGUIObj->MultiSelection = MultiSelection;
    pGUIObj->Item = NULL;
    pGUIObj->EventHandler = pEventHandler;

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_LISTBOX;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }

  return pPageObj;
}

/**
  * @brief  Creates and initializse a RadioButton Group object
  * @param  ID:        object ID.
  * @retval GL_RadioButtonGrp_TypeDef* - The created Object pointer
  */
GL_RadioButtonGrp_TypeDef* NewRadioButtonGrp (uint16_t ID)
{
  GL_RadioButtonGrp_TypeDef *pRButtonGrp = NULL;

  pRButtonGrp = (GL_RadioButtonGrp_TypeDef *)malloc(sizeof(GL_RadioButtonGrp_TypeDef));

  if (pRButtonGrp)
  {
    pRButtonGrp->ID = ID;
    /* Create the RadioButton object */
    Create_RadioButtonGrp(pRButtonGrp);
  }
  return pRButtonGrp;
}

/**
  * @brief  Creates and initializes a Slidebar object
  * @param  ID:        object ID.
  * @param  Label:     label for the Slidebar object.
  * @param  Direction: the printing orientation.
  *         This parameter can be one of the following values:
  *     @arg  GL_Vertical
  *     @arg  GL_Horizontal
  * @param  pEventHandler: pointer to function associated to its touch event.
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewSlidebar (uint16_t ID,
                                      const char* Label,
                                      GL_Direction Direction,
                                      void (*pEventHandler)(void))
{
  GL_Slidebar_TypeDef *pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr;

  pGUIObj = (GL_Slidebar_TypeDef *)malloc(sizeof(GL_Slidebar_TypeDef));

  if ( pGUIObj )
  {
    pStr = malloc(strlen((char*)Label));
    if ( strlen((char*)Label) > MAX_SLIDE_LABEL_LENGTH )
    {
      // Only the first MAX_SLIDE_LABEL_LENGTH characters of the slidebar label can be printed
      strncpy(pStr,(char*)Label, MAX_SLIDE_LABEL_LENGTH);
      pStr[MAX_SLIDE_LABEL_LENGTH] = '\0';   // Null character manually added
    }
    else
    {
      strcpy(pStr,(char*)Label);
    }

    pGUIObj->ID = ID;
    pGUIObj->Label = (uint8_t*)pStr;
    pGUIObj->Direction = Direction;
    pGUIObj->Width = 18;
    pGUIObj->Length = 118;
    pGUIObj->CurrentValue = 0;
    pGUIObj->PreviousValue = 0;
    pGUIObj->MinValue = 0;
    pGUIObj->MaxValue = 100;
    pGUIObj->EventHandler = pEventHandler;

    // Create the Slidebar object
    Create_Slidebar(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_SLIDEBAR;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a Switch Button object
  * @param  ID:            object ID.
  * @param  Label_1:       text on unclicked switch button.
  * @param  Label_2:       text on clicked switch button.
  * @param  pEventHandler: pointer to function associated to its touch event.
  * @retval pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewSwitch (uint16_t ID,
                                    const char* Label_1,
                                    const char* Label_2,
                                    void (*pEventHandler)(void))
{
  GL_Switch_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;
  char* pStr1;
  char* pStr2;

  pGUIObj = (GL_Switch_TypeDef*)malloc(sizeof(GL_Switch_TypeDef));

  if ( pGUIObj )
  {
    pStr1 = malloc(strlen((char*)Label_1));
    strcpy(pStr1,(char*)Label_1);
    pStr2 = malloc(strlen((char*)Label_2));
    strcpy(pStr2,(char*)Label_2);

    pGUIObj->ID = ID;
    pGUIObj->Label_1 = (uint8_t*)pStr1;
    pGUIObj->Label_2 = (uint8_t*)pStr2;
    pGUIObj->EventHandler = pEventHandler;

    // Create the Switch object
    Create_Switch(pGUIObj);

    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));
    if ( pPageObj )
    {
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_SWITCH;
    }
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  return pPageObj;
}

/**
  * @brief  Creates and initializes a textbox object.
  * @param  ID:       object ID.
  * @param  Width:    textbox width.
  * @param  NrOfRows: number of rows in textbox.
  * @retval *pPageObj - the pointer to the graphic object.
  */
GL_PageControls_TypeDef* NewTextbox (uint16_t ID,
                                     uint16_t Width,
                                     uint16_t NrOfRows)
{
  GL_Textbox_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pPageObj = NULL;

  // Sprawdz czy podane wymiary komponentu sa prawidlowe
  // (czy komponent moze wyswietlic co najmniej jeden symbol)
  if ( (NrOfRows < 1) || (Width < SMALL_FONT_WIDTH+6) )
  {
    // Jezeli nie to zwroc NULL
    return NULL;
  }

  // Zarezerwuj wymagana przez konkretny obiekt ilosc miejsca w pamieci RAM
  pGUIObj = (GL_Textbox_TypeDef*)malloc(sizeof(GL_Textbox_TypeDef));

  // Jesli rezerwacja powiodla sie
  if ( pGUIObj )
  {
    // Wypelnij pola struktury
    pGUIObj->ID = ID;
    pGUIObj->Width = Width;
    pGUIObj->Height = SMALL_FONT_HEIGHT*NrOfRows+4;
    pGUIObj->Text = NULL;

    // Zarezerwuj wymagana przez ogolny obiekt ilosc miejsca w pamieci RAM
    pPageObj = (GL_PageControls_TypeDef*)malloc(sizeof(GL_PageControls_TypeDef));

    // Jesli rezerwacja powiodla sie
    if ( pPageObj )
    {
      // Zapisz wskaznik na konkretny obiekt i jego rodzaj
      pPageObj->objPTR = (void*)pGUIObj;
      pPageObj->objType = GL_TEXTBOX;
    }
    // W przeciwnym wypadku zwolnij miejsce w pamieci RAM
    else
    {
      free(pGUIObj);
      pGUIObj = NULL;
    }
  }
  // Zwroc wskaznik na obiekt
  return pPageObj;
}


/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetButtonVisible (GL_PageControls_TypeDef* pObj,
                                      GL_Coordinates_TypeDef objCoordinates)
{
  GL_Button_TypeDef* pGUIObj = (GL_Button_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  uint8_t labelLength;

#ifndef USE_2D_OBJECTS
  uint16_t maxY = objCoordinates.MaxY;

  uint16_t counter;
  uint8_t* ptrBitmapLeft = NULL;
  uint8_t* ptrBitmapCenter = NULL;
  uint8_t* ptrBitmapRight = NULL;
#else
  uint16_t color = 0;
#endif

  // Check if pGUIObj == NULL
  if(!pGUIObj)
  {
    return GL_ERROR;
  }

  labelLength = strlen((char*)pGUIObj->Label);

  pGUIObj->Control_Visible = GL_TRUE;

  if ( pGUIObj->ObjectTouched == GL_FALSE )
  {
    GL_SetTextColor(BUTTON_NORMAL_TEXT_COLOR);
#ifdef USE_2D_OBJECTS
    color = BUTTON_NORMAL_BG_COLOR;
#else
    ptrBitmapLeft = (uint8_t*)BtnNormalLeft;
    ptrBitmapCenter = (uint8_t*)BtnNormal;
    ptrBitmapRight = (uint8_t*)BtnNormalRight;
#endif
  }
  else if ( pGUIObj->ObjectTouched == GL_TRUE )
  {
    GL_SetTextColor(BUTTON_PRESSED_TEXT_COLOR);
#ifdef USE_2D_OBJECTS
    color = BUTTON_PRESSED_BG_COLOR;
#else
    ptrBitmapLeft = (uint8_t*)BtnPressedLeft;
    ptrBitmapCenter = (uint8_t*)BtnPressed;
    ptrBitmapRight = (uint8_t*)BtnPressedRight;
#endif
  }

#ifdef USE_2D_OBJECTS
  GL_DrawFilledRectangle( maxX,
                          minY,
                          labelLength*SMALL_FONT_WIDTH + 2*BUTTON_PIECE_LENGTH,
                          BUTTON_HEIGHT,
                          color);
#else
  // Rysuj lewa czesc przycisku
  GL_DrawColorBMP( ptrBitmapLeft, maxX, maxY, BUTTON_HEIGHT, BUTTON_PIECE_LENGTH);

  // Rysuj centralna czesc przycisku tyle razy ile jest symboli w etykiecie
  for (counter = 1; counter < labelLength + 1; counter++)
  {
    GL_DrawColorBMP( ptrBitmapCenter,
                     maxX - (counter*BUTTON_PIECE_LENGTH),
                     maxY,
                     BUTTON_HEIGHT,
                     BUTTON_PIECE_LENGTH );
  }

  // Rysuj prawa czesc przycisku
  GL_DrawColorBMP( ptrBitmapRight,
                   maxX - (counter*BUTTON_PIECE_LENGTH),
                   maxY,
                   BUTTON_HEIGHT,
                   BUTTON_PIECE_LENGTH );
#endif

  GL_SetFont(GL_FONT_SMALL);

  // Wyswietl etykiete przycisku
  GL_DrawString( maxX - BUTTON_PIECE_LENGTH,
                 minY + ((BUTTON_HEIGHT - GL_FontHeight) / 2) + 1,
                 (uint8_t*) pGUIObj->Label, GL_TRUE);

  GL_SetTextColor(GL_Black);
  return GL_OK;
}

static GL_ErrStatus SetButtonColorVisible (GL_PageControls_TypeDef* pObj,
                                           GL_Coordinates_TypeDef objCoordinates)
{
  GL_ButtonColor_TypeDef* pGUIObj = (GL_ButtonColor_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  uint16_t color = 0;

  // Check if pGUIObj == NULL
  if(!pGUIObj)
  {
    return GL_ERROR;
  }

  color = pGUIObj->Color;

  pGUIObj->Control_Visible = GL_TRUE;

  GL_SetForegroundColor(Black);
  GL_DrawRectangle( maxX, minY, 28, 28);
  GL_DrawFilledRectangle( maxX-1, minY+1, 26, 26, color);

  return GL_OK;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure.
  * @param  objCoordinates: pointer to objCoordinates Structure.
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetCheckboxVisible (GL_PageControls_TypeDef* pObj,
                                        GL_Coordinates_TypeDef objCoordinates)
{
  GL_Checkbox_TypeDef* pGUIObj = (GL_Checkbox_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

#ifndef USE_2D_OBJECTS
  uint16_t maxY = objCoordinates.MaxY;

  uint8_t* ptrBitmap;
#endif

  // Check if pGUIObj == NULL
  if(!pGUIObj)
  {
    return GL_ERROR;
  }

  //pGUIObj->Control_Visible = GL_TRUE;//?

#ifdef USE_2D_OBJECTS
  GL_DrawFilledRectangle( maxX,
                          minY,
                          CHECKBOX_SIZE,
                          CHECKBOX_SIZE,
                          GL_Black);

  GL_DrawFilledRectangle( maxX - 1,
                          minY + 1,
                          CHECKBOX_SIZE - 2,
                          CHECKBOX_SIZE - 2,
                          GL_White);
#endif

  if ( pGUIObj->Checked == GL_FALSE )
  {
#ifndef USE_2D_OBJECTS
    ptrBitmap = (uint8_t*)CheckBox1;
#endif
  }
  else
  {
#ifndef USE_2D_OBJECTS
    ptrBitmap = (uint8_t*)CheckBox2;
#else
    GL_SetTextColor(GL_Black);
    GL_DrawString( maxX - ((CHECKBOX_SIZE - GL_FontWidth) / 2),
                   minY + ((CHECKBOX_SIZE - GL_FontHeight) / 2) + 1,
                   (uint8_t*) "X", GL_TRUE);
#endif
  }

#ifndef USE_2D_OBJECTS
  GL_DrawColorBMP( ptrBitmap, maxX, maxY, CHECKBOX_SIZE, CHECKBOX_SIZE);
#endif

  GL_SetFont(GL_FONT_SMALL);
  GL_SetTextColor(GL_Black);

  // Wyswietl etykiete checkboxa
  GL_DrawString( maxX - (CHECKBOX_SIZE + 1),
                 minY + ((CHECKBOX_SIZE - GL_FontHeight) / 2) + 1,
                 (uint8_t*) pGUIObj->Label, GL_TRUE);

  GL_SetTextColor(GL_Black);
  return GL_OK;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetLabelVisible (GL_PageControls_TypeDef* pObj,
                                     GL_Coordinates_TypeDef objCoordinates)
{
  GL_Label_TypeDef* pGUIObj = (GL_Label_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  // Check if pGUIObj == NULL
  if ( !pGUIObj )
  {
    return GL_ERROR;
  }

  pGUIObj->Control_Visible = GL_TRUE;

  GL_SetTextColor(pGUIObj->Colour);
  GL_SetFont(pGUIObj->FontSize);

  if ( pGUIObj->Direction == GL_HORIZONTAL )
  {
    GL_ChangeDirection(_0_degree);
  }
  else if ( pGUIObj->Direction == GL_LEFT_VERTICAL )
  {
    GL_ChangeDirection(_90_degree);
  }
  else if ( pGUIObj->Direction == GL_RIGHT_VERTICAL )
  {
    GL_ChangeDirection(_270_degree);
  }

  GL_DrawString(maxX, minY, pGUIObj->Label, GL_TRUE);

  GL_ChangeDirection(_0_degree);
  GL_SetFont(GL_FONT_SMALL);
  GL_SetTextColor(GL_Black);
  return GL_OK;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetRadioButtonVisible (GL_PageControls_TypeDef* pObj,
                                           GL_Coordinates_TypeDef objCoordinates)
{
  GL_RadioOption_TypeDef* pGUIObj = (GL_RadioOption_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

#ifndef USE_2D_OBJECTS
  uint16_t maxY = objCoordinates.MaxY;

  uint8_t* ptrBitmap;
#endif

  if ( !pGUIObj )
  {
    return GL_ERROR;
  }

  pGUIObj->Control_Visible = GL_TRUE;

  if ( pGUIObj->Checked == GL_TRUE )
  {
#ifdef USE_2D_OBJECTS
    GL_SetForegroundColor(GL_Black);
    GL_DrawFilledCircle( (uint16_t)(objCoordinates.MaxX), (uint8_t)(objCoordinates.MaxY), RADIO_BUTTON_RADIUS, GL_White );
    GL_DrawFilledCircle( (uint16_t)(objCoordinates.MaxX-(RADIO_BUTTON_RADIUS/2 -1)),
                         (uint8_t)(objCoordinates.MaxY-(RADIO_BUTTON_RADIUS/2 -1)),
                         RADIO_BUTTON_RADIUS/2 +2, GL_Black);
#else
    ptrBitmap = (uint8_t*)RButtonA;
#endif
  }
  else
  {
#ifdef USE_2D_OBJECTS
    GL_SetForegroundColor(GL_Black);
    GL_DrawFilledCircle((uint16_t)(objCoordinates.MaxX), (uint8_t)(objCoordinates.MaxY), RADIO_BUTTON_RADIUS, GL_White);
    GL_DrawFilledRectangle( maxX - (RADIO_BUTTON_RADIUS/2) - 1,
                            minY + (RADIO_BUTTON_RADIUS/2) + 2,
                            RADIO_BUTTON_RADIUS,
                            RADIO_BUTTON_RADIUS,
                            GL_White);
#else
    ptrBitmap = (uint8_t*)RButtonB;
#endif
  }

#ifndef USE_2D_OBJECTS
  GL_DrawColorBMP( ptrBitmap, maxX, maxY, RBUTTON_OPT_SIZE, RBUTTON_OPT_SIZE);
#endif

  GL_SetFont(GL_FONT_SMALL);
  GL_SetTextColor(GL_Black);

  // Wyswietl etykiete radiobuttona
  GL_DrawString( maxX - (RBUTTON_OPT_SIZE + 2),
                 minY + ((RBUTTON_OPT_SIZE - GL_FontHeight) / 2),
                 (uint8_t*) pGUIObj->Label, GL_TRUE);

  return GL_OK;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetSlidebarVisible (GL_PageControls_TypeDef* pObj,
                                        GL_Coordinates_TypeDef objCoordinates)
{
  GL_Slidebar_TypeDef* pGUIObj = (GL_Slidebar_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minX = objCoordinates.MinX;
  uint16_t maxY = objCoordinates.MaxY;
  uint16_t minY = objCoordinates.MinY;

  uint8_t labelLength = 0;

#ifndef USE_2D_OBJECTS
  uint16_t counter;
  uint8_t* ptrBitmapLeft = (uint8_t*)SlidebarLeft;
  uint8_t* ptrBitmapCenter = (uint8_t*)SlidebarCentral;
  uint8_t* ptrBitmapRight = (uint8_t*)SlidebarRight;
  uint8_t* ptrBitmapCursor = (uint8_t*)SlidebarCursor;
#endif

  // Check if pGUIObj == NULL
  if(!pGUIObj)
  {
    return GL_ERROR;
  }

  labelLength = strlen((char*)pGUIObj->Label);

  pGUIObj->Control_Visible = GL_TRUE;

  if ( pGUIObj->Direction == GL_HORIZONTAL )
  {
    // Cleaning the Slidebar main body contour
    GL_DrawFilledRectangle( maxX, minY - 1, pGUIObj->Length, pGUIObj->Width + 2, GL_White);
#ifdef USE_2D_OBJECTS
    // Drawing the Slidebar main body
    GL_DrawFilledRectangle( maxX, minY, pGUIObj->Length, pGUIObj->Width, GL_Blue);
#else
    GL_DrawColorBMP( ptrBitmapLeft, maxX, maxY, SLIDEBAR_HEIGHT, SLIDEBAR_PIECE_LENGTH);

    for (counter = 1; counter < SLIDEBAR_CENTRAL_LENGTH; counter++)
    {
      GL_DrawColorBMP( ptrBitmapCenter,
    		           maxX - (counter*SLIDEBAR_PIECE_LENGTH),
    		           maxY,
    		           SLIDEBAR_HEIGHT,
    		           SLIDEBAR_PIECE_LENGTH );
    }

    GL_DrawColorBMP( ptrBitmapRight,
    		          maxX - (counter*SLIDEBAR_PIECE_LENGTH),
                      maxY,
                      SLIDEBAR_HEIGHT,
   		              SLIDEBAR_PIECE_LENGTH );
#endif
  }
  else if ( pGUIObj->Direction == GL_LEFT_VERTICAL )
  {
    // Cleaning the Slidebar main body contour
    GL_DrawFilledRectangle( maxX + 1, minY, pGUIObj->Width + 2, pGUIObj->Length, GL_White);
#ifdef USE_2D_OBJECTS
    // Drawing the SlideBar main body
    GL_DrawFilledRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Length, GL_Blue);
#else
    GL_ChangeDirection(_90_degree);

    LCD_DrawColorBMP( ptrBitmapLeft, minX, maxY, SLIDEBAR_HEIGHT, SLIDEBAR_PIECE_LENGTH);

    for (counter = 1; counter < SLIDEBAR_CENTRAL_LENGTH; counter++)
    {
      LCD_DrawColorBMP( ptrBitmapCenter,
                        minX,
                        maxY - (counter*SLIDEBAR_PIECE_LENGTH),
                        SLIDEBAR_HEIGHT,
                        SLIDEBAR_PIECE_LENGTH);
    }

    LCD_DrawColorBMP( ptrBitmapRight,
                      minX,
                      maxY - (counter*SLIDEBAR_PIECE_LENGTH),
                      SLIDEBAR_HEIGHT,
                      SLIDEBAR_PIECE_LENGTH);
#endif
  }
  else if ( pGUIObj->Direction == GL_RIGHT_VERTICAL )
  {
    // Cleaning the Slidebar main body contour
    GL_DrawFilledRectangle( maxX/* - GL_FontHeight - 1*/, minY, pGUIObj->Width + 2, pGUIObj->Length, GL_White);
#ifdef USE_2D_OBJECTS
    // Drawing the SlideBar main body
    GL_DrawFilledRectangle( maxX/* - GL_FontHeight - 1*/ - 1, minY, pGUIObj->Width, pGUIObj->Length, GL_Blue);
#else
    GL_ChangeDirection(_270_degree);

    minY += SLIDEBAR_PIECE_LENGTH;

    LCD_DrawColorBMP( ptrBitmapLeft, (maxX - 1/* - GL_FontHeight - 1*/), minY, SLIDEBAR_HEIGHT, SLIDEBAR_PIECE_LENGTH);

    for (counter = 1; counter < SLIDEBAR_CENTRAL_LENGTH; counter++)
    {
      LCD_DrawColorBMP( ptrBitmapCenter,
    		            maxX - 1/*GL_FontHeight - 1*/,
                        minY + (counter*SLIDEBAR_PIECE_LENGTH),
                        SLIDEBAR_HEIGHT,
                        SLIDEBAR_PIECE_LENGTH);
    }

    LCD_DrawColorBMP( ptrBitmapRight,
    		          maxX - 1/* - GL_FontHeight - 1*/,
                      minY + (counter*SLIDEBAR_PIECE_LENGTH),
                      SLIDEBAR_HEIGHT,
                      SLIDEBAR_PIECE_LENGTH);
#endif
  }

  // Drawing the SlideBar cursor
  if ( pGUIObj->Direction == GL_HORIZONTAL )
  {
#ifdef USE_2D_OBJECTS
    GL_DrawFilledRectangle( maxX - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH,
                            minY - 1,
                            SLIDEBAR_CURSOR_LENGTH,
                            pGUIObj->Width + 2,
                            GL_Green);

    GL_SetForegroundColor(GL_Grey);
    GL_DrawRectangle( maxX - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH,
                      minY - 1,
                      SLIDEBAR_CURSOR_LENGTH,
                      pGUIObj->Width + 2);
#else
    LCD_DrawColorBMP( ptrBitmapCursor,
                      maxX - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH,
                      maxY + 1,
                      SLIDEBAR_HEIGHT + 2,
                      SLIDEBAR_CURSOR_LENGTH );
#endif
  }
  else if ( pGUIObj->Direction == GL_LEFT_VERTICAL )
  {
#ifdef USE_2D_OBJECTS
    GL_DrawFilledRectangle( maxX + 1,
                            minY + pGUIObj->Length - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH - SLIDEBAR_CURSOR_LENGTH,
                            pGUIObj->Width + 2,
                            SLIDEBAR_CURSOR_LENGTH,
                            GL_Green);

    GL_SetForegroundColor(GL_Grey);
    GL_DrawRectangle( maxX + 1,
                      minY + pGUIObj->Length - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH - SLIDEBAR_CURSOR_LENGTH,
                      pGUIObj->Width + 2,
                      SLIDEBAR_CURSOR_LENGTH);
#else
    GL_ChangeDirection(_90_degree);

    LCD_DrawColorBMP( ptrBitmapCursor,
                      minX - 1,
                      maxY - pGUIObj->CurrentValue - SLIDEBAR_OFFSET_LENGTH,
                      SLIDEBAR_HEIGHT + 2,
                      SLIDEBAR_CURSOR_LENGTH);
#endif
  }
  else if ( pGUIObj->Direction == GL_RIGHT_VERTICAL )
  {
#ifdef USE_2D_OBJECTS
    GL_DrawFilledRectangle( maxX/* - GL_FontHeight - 1*/,
                            minY + pGUIObj->CurrentValue + SLIDEBAR_OFFSET_LENGTH,
                            pGUIObj->Width + 2,
                            SLIDEBAR_CURSOR_LENGTH,
                            GL_Green);

    GL_SetForegroundColor(GL_Grey);
    GL_DrawRectangle( maxX/* - GL_FontHeight - 1*/,
                      minY + pGUIObj->CurrentValue + SLIDEBAR_OFFSET_LENGTH,
                      pGUIObj->Width + 2,
                      SLIDEBAR_CURSOR_LENGTH);
#else
    GL_ChangeDirection(_270_degree);

    LCD_DrawColorBMP( ptrBitmapCursor,
                      maxX/* + 1 - GL_FontHeight - 2*/,
                      minY + pGUIObj->CurrentValue + SLIDEBAR_OFFSET_LENGTH,
                      SLIDEBAR_HEIGHT + 2,
                      SLIDEBAR_CURSOR_LENGTH);
#endif
  }

  pGUIObj->PreviousValue = pGUIObj->CurrentValue;

  if ( labelLength > 0 )
  {
    GL_SetTextColor(GL_Black);
    GL_SetFont(GL_FONT_SMALL);

    if ( pGUIObj->Direction == GL_HORIZONTAL )
    {
      maxX = maxX - ((pGUIObj->Length - labelLength*GL_FontWidth) / 2) + 1;
      minY = maxY + 2;
    }
    else if ( pGUIObj->Direction == GL_LEFT_VERTICAL )
    {
      GL_ChangeDirection(_90_degree);
      maxX = minX - 2;
      minY = minY + ((pGUIObj->Length - labelLength*GL_FontWidth) / 2) - 1;
    }
    else if ( pGUIObj->Direction == GL_RIGHT_VERTICAL )
    {
      GL_ChangeDirection(_270_degree);
      maxX = maxX + GL_FontHeight + 1;//?
      minY = minY + ((pGUIObj->Length - labelLength*GL_FontWidth) / 2) - 1;
    }

    // Wyswietl tekst pod suwakiem
    GL_DrawString(maxX, minY, pGUIObj->Label, GL_TRUE);
  }

  GL_ChangeDirection(_0_degree);
  GL_SetFont(GL_FONT_SMALL);
  GL_SetTextColor(GL_Black);
  return GL_OK;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetSwitchVisible (GL_PageControls_TypeDef* pObj,
                                      GL_Coordinates_TypeDef objCoordinates)
{
  GL_Switch_TypeDef* pGUIObj = (GL_Switch_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  uint8_t maxLabelLength = 0;
  uint8_t* currentLabel;

#ifndef USE_2D_OBJECTS
  uint16_t maxY = objCoordinates.MaxY;

  uint16_t counter;
  uint8_t* ptrBitmapLeft = NULL;
  uint8_t* ptrBitmapCenter = NULL;
  uint8_t* ptrBitmapRight = NULL;
#else
  uint16_t color;
#endif

  // Check if pGUIObj == NULL
  if(!pGUIObj)
  {
    return GL_ERROR;
  }

  if ( strlen( (char*)(pGUIObj->Label_1) ) > strlen( (char*)(pGUIObj->Label_2) ) )
  {
    maxLabelLength = strlen( (char*)(pGUIObj->Label_1) );
  }
  else
  {
    maxLabelLength = strlen( (char*)(pGUIObj->Label_2) );
  }

  if ( pGUIObj->ObjectTouched == GL_FALSE )
  {
    currentLabel = pGUIObj->Label_1;
    GL_SetTextColor(SWITCH_STATE_OFF_TEXT_COLOR);
#ifdef USE_2D_OBJECTS
    color = SWITCH_STATE_OFF_BG_COLOR;
#else
    ptrBitmapLeft = (uint8_t*)SwitchPressedLeft;
    ptrBitmapCenter = (uint8_t*)SwitchPressed;
    ptrBitmapRight = (uint8_t*)SwitchPressedRight;
#endif
  }
  else // pGUIObj->ObjectTouched == GL_TRUE
  {
    currentLabel = pGUIObj->Label_2;
    GL_SetTextColor(SWITCH_STATE_ON_TEXT_COLOR);
#ifdef USE_2D_OBJECTS
    color = SWITCH_STATE_ON_BG_COLOR;
#else
    ptrBitmapLeft = (uint8_t*)SwitchNormalLeft;
    ptrBitmapCenter = (uint8_t*)SwitchNormal;
    ptrBitmapRight = (uint8_t*)SwitchNormalRight;
#endif
  }

#ifdef USE_2D_OBJECTS
  GL_DrawFilledRectangle( maxX,
                          minY,
                          maxLabelLength*SMALL_FONT_WIDTH + 2*BUTTON_PIECE_LENGTH,
                          BUTTON_HEIGHT,
                          color);
#else
  // Rysuj lewa czesc przelacznika
  GL_DrawColorBMP( ptrBitmapLeft, maxX, maxY, BUTTON_HEIGHT, BUTTON_PIECE_LENGTH);

  // Rysuj centralna czesc przelacznika tyle razy ile jest symboli w etykiecie
  for (counter = 1; counter < maxLabelLength + 1; counter++)
  {
    GL_DrawColorBMP( ptrBitmapCenter,
  		             maxX - (counter*BUTTON_PIECE_LENGTH),
  		             maxY,
  		             BUTTON_HEIGHT,
  		             BUTTON_PIECE_LENGTH );
  }

  // Rysuj prawa czesc przelacznika
  GL_DrawColorBMP( ptrBitmapRight,
  		           maxX - (counter*BUTTON_PIECE_LENGTH),
                   maxY,
                   BUTTON_HEIGHT,
                   BUTTON_PIECE_LENGTH );
#endif

  GL_SetFont(GL_FONT_SMALL);

  // Wyswietl etykiete przelacznika
  GL_DrawString( maxX - BUTTON_PIECE_LENGTH,
                 minY + ((BUTTON_HEIGHT - GL_FontHeight) / 2) + 1,
                 (uint8_t*) currentLabel, GL_TRUE);

  GL_SetTextColor(GL_Black);
  return GL_OK;
}

/**
  * @brief  Modifies the text in a label object
  * @param  pPage: pointer to Page Structure
  * @param  ID:    object identifier
  * @param  text:  Text on a Label Object
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus SetLabelText (GL_Page_TypeDef* pPage,
                           uint16_t ID,
                           const char* text)
{
  GL_Label_TypeDef* pGUIObj = NULL;
  char* pStr = NULL;

  uint16_t temp = 0;
  uint16_t index = 0;

  if ( !pPage )
  {
    return GL_ERROR;
  }

  if (strlen(text) > 0)
  {
    /* search for the required label */
    while ( index < pPage->ControlCount )
    {
      if (pPage->PageControls[index]->objType == GL_LABEL)
      {
        pGUIObj = (GL_Label_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pGUIObj->ID == ID)
        {
          // Hide previous text
          temp = pGUIObj->Colour;
          pGUIObj->Colour = GL_GetBackgroundColor();
          RefreshPageControl(pPage, ID);//?

          free(pGUIObj->Label);

          pStr = malloc(strlen(text));
          strcpy(pStr,text);

          pGUIObj->Label = (uint8_t*)pStr;

          // Show new text
          pGUIObj->Colour = temp;
          RefreshPageControl(pPage, ID);//?

          return GL_OK;
        }
      }
      index++;
    }
  }
  return GL_ERROR;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pObj:           pointer to Object Structure
  * @param  objCoordinates: pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetListboxVisible (GL_PageControls_TypeDef* pObj,
                                       GL_Coordinates_TypeDef objCoordinates)
{
  // Get pointer to GUI object
  GL_Listbox_TypeDef* pGUIObj = (GL_Listbox_TypeDef*)(pObj->objPTR);

  uint16_t color, i, y_pos;
  uint16_t slipper_length, slipper_length_max;
  int16_t nrInvisibleItems, nrVisibleItems;

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minX = objCoordinates.MinX;
  uint16_t maxY = objCoordinates.MaxY;
  uint16_t minY = objCoordinates.MinY;

  if (!pGUIObj)
  {
    return GL_ERROR;
  }

  GL_SetForegroundColor(GL_Black);

  // Clear Listbox area
  GL_DrawFilledRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height, GL_White );

  // Draw box
  GL_DrawRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height);

  y_pos = minY + 2;

  // Draw UP arrow of vertical scrollbar
  GL_DrawRectangle( minX + 2 + SCROLLBAR_THICKNESS - 1, y_pos, SCROLLBAR_THICKNESS, SCROLLBAR_ARROW_LENGTH );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS/2 + 1, minY + 4, minX + 4, minY + SCROLLBAR_ARROW_LENGTH - 1 );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS/2 + 2, minY + 4, minX + SCROLLBAR_THICKNESS - 1, minY + SCROLLBAR_ARROW_LENGTH - 1 );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS - 1, minY + SCROLLBAR_ARROW_LENGTH - 1, minX + 4, minY + SCROLLBAR_ARROW_LENGTH - 1 );

  y_pos += (SCROLLBAR_ARROW_LENGTH - 1);

  // Draw vertical scrollbar
  GL_DrawRectangle( minX + 2 + SCROLLBAR_THICKNESS - 1, y_pos, SCROLLBAR_THICKNESS, pGUIObj->Height - (SCROLLBAR_ARROW_LENGTH*2 + 2) );

  y_pos = minY + (pGUIObj->Height - (SCROLLBAR_ARROW_LENGTH + 2));

  // Draw DOWN arrow of vertical scrollbar
  GL_DrawRectangle( minX + 2 + SCROLLBAR_THICKNESS - 1, y_pos, SCROLLBAR_THICKNESS, SCROLLBAR_ARROW_LENGTH );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS/2 + 1, maxY - 4, minX + 4, maxY - SCROLLBAR_ARROW_LENGTH + 1 );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS/2 + 2, maxY - 4, minX + SCROLLBAR_THICKNESS - 1, maxY - SCROLLBAR_ARROW_LENGTH + 1 );
  GL_DrawLine_2( minX + SCROLLBAR_THICKNESS - 1, maxY - SCROLLBAR_ARROW_LENGTH + 1, minX + 4, maxY - SCROLLBAR_ARROW_LENGTH + 1 );

  // Calculate maximum number of visible items
  nrVisibleItems = (uint16_t) ( (pGUIObj->Height - 4) / SMALL_FONT_HEIGHT );

  // Calculate number of invisible items
  nrInvisibleItems = (int16_t) (pGUIObj->NrOfItems - nrVisibleItems);

  // If all items are visible
  if (nrInvisibleItems < 0)
  {
    nrInvisibleItems = 0;
    nrVisibleItems = pGUIObj->NrOfItems;
  }

  // Calculate slipper maximum length
  slipper_length_max = (pGUIObj->Height - (SCROLLBAR_ARROW_LENGTH*2 + 4));
  // Calculate slipper real length
  slipper_length = (uint16_t) (slipper_length_max * (1 - (((float) nrInvisibleItems )/(pGUIObj->NrOfItems)) ));

  // Calculate slipper position
  y_pos = minY + SCROLLBAR_ARROW_LENGTH + 2 + (uint16_t) (pGUIObj->NrOfFirstItem*(  (float) slipper_length_max / (pGUIObj->NrOfItems) ));

  // Clear slipper background
  GL_DrawFilledRectangle( minX + 2 + SCROLLBAR_THICKNESS - 2, minY + SCROLLBAR_ARROW_LENGTH + 2, SCROLLBAR_THICKNESS - 2, pGUIObj->Height - (SCROLLBAR_ARROW_LENGTH*2 + 4), GL_White );

  if(nrInvisibleItems > 0)
  {
    // Draw slipper on vertical scrollbar
    GL_DrawFilledRectangle( minX + 2 + SCROLLBAR_THICKNESS - 2, y_pos, SCROLLBAR_THICKNESS - 2, slipper_length, GL_Blue );
  }

  if( (pGUIObj->NrOfItems) > 0)
  {
    GL_SetFont(GL_FONT_SMALL);

    // Find first item in list
    while (pGUIObj->Item->PrevItem != NULL)
    {
      pGUIObj->Item = pGUIObj->Item->PrevItem;
    }

    // Find first visible item in list
    for(i = 0; i < pGUIObj->NrOfFirstItem; i++)
    {
      if(pGUIObj->Item->NextItem != NULL)
      {
        pGUIObj->Item = pGUIObj->Item->NextItem;
      }
    }

    // Print visible items
    for (i = 0; i < nrVisibleItems; i++)
    {
      if(pGUIObj->Item->ItemSelected == GL_TRUE)
      {
         color = GL_Blue;
         GL_SetTextColor(GL_White);
      }
      else
      {
         color = GL_White;
         GL_SetTextColor(GL_Black);
      }

      // Draw item background
      GL_DrawFilledRectangle( maxX - 2, minY + 2 + SMALL_FONT_HEIGHT*i, pGUIObj->Width - (SCROLLBAR_THICKNESS + 5), SMALL_FONT_HEIGHT, color );
      // Draw item text
      GL_DrawString( maxX - 3, minY + 2 + SMALL_FONT_HEIGHT*i, pGUIObj->Item->Text, GL_TRUE);

      // If last item printed
      if( !(i < nrVisibleItems-1) )
      {
         break;
      }

      pGUIObj->Item = pGUIObj->Item->NextItem;
    }
  }

  GL_SetTextColor(GL_Black);

  return GL_OK;
}

/**
  * @brief  Show the Control Object at the specified coordinates.
  * @param  *pThis: Pointer to Object Structure
  * @param  *objCoordinates: Pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetComboBoxVisible (GL_PageControls_TypeDef* pObj,
                                        GL_Coordinates_TypeDef objCoordinates)
{
  GL_ComboBoxGrp_TypeDef* pGUIObj = (GL_ComboBoxGrp_TypeDef*)(pObj->objPTR);

  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minX = objCoordinates.MinX;
  uint16_t minY = objCoordinates.MinY;

#ifndef USE_2D_OBJECTS
  uint16_t maxY = objCoordinates.MaxY;

  uint8_t* ptrBitmap = pGUIObj->ImageUnClickedPTR;
#endif

  if (!pGUIObj)
  {
    return GL_ERROR;
  }




  uint32_t index = 0;

  pGUIObj->Control_Visible = GL_TRUE;

  GL_SetFont(GL_FONT_SMALL);
  GL_SetBackgroundColor(GL_White);
  GL_SetTextColor(GL_Black);













#ifdef USE_2D_OBJECTS
  GL_DrawFilledRectangle( maxX,
                          minY,
                          COMBOBOX_SIZE,
                          COMBOBOX_SIZE,
                          GL_Grey);
#else
  GL_DrawColorBMP( ptrBitmap,
                   maxX,
                   maxY,
                   COMBOBOX_SIZE,
                   COMBOBOX_SIZE);
#endif

  GL_SetTextColor(GL_Black);
  GL_SetForegroundColor(GL_Black);










  // Jezeli obiekt reprezentuje strzalke w gore
  if (pGUIObj->Secondary == GL_FALSE)
  {
#ifdef USE_2D_OBJECTS
    GL_SetFont(GL_FONT_BIG);
    GL_DisplayAdjStringLine( (uint8_t)(objCoordinates.MinY-1), (uint16_t)(objCoordinates.MaxX-1), "-", GL_TRUE);
    GL_DisplayAdjStringLine( (uint8_t)(objCoordinates.MinY-1), (uint16_t)(objCoordinates.MaxX-4), "-", GL_TRUE);
#endif
    for (; index < pGUIObj->ComboOptionCount && pGUIObj->ComboOptions[index]->Active == GL_FALSE; index++)
    {}
    GL_DrawFilledRectangle( minX - COMBOBOX_SIZE - 1,
                            minY,
                            MAX_COMBO_LABEL_LENGTH*FONT_LENGTH,
                            COMBOBOX_SIZE,
                            GL_White);
    GL_DrawRectangle( minX - COMBOBOX_SIZE - 1,
                      minY,
                      MAX_COMBO_LABEL_LENGTH*FONT_LENGTH,
                      COMBOBOX_SIZE);

    GL_SetFont(GL_FONT_SMALL);
    if(pGUIObj->ComboOptionCount>0)
    {
      GL_DisplayAdjStringLine( (uint8_t)(objCoordinates.MaxY - 16),
                               (uint16_t)(objCoordinates.MaxX) - (COMBOBOX_SIZE*2 + 4),
                               (char*)pGUIObj->ComboOptions[index]->Label, GL_TRUE);
    }
  }
  // W przeciwnym wypadku obiekt reprezentuje strzalke w dol
  else
  {
#ifdef USE_2D_OBJECTS
    GL_SetFont(GL_FONT_BIG);
    GL_DisplayAdjStringLine( (uint8_t)(objCoordinates.MinY), (uint16_t)(objCoordinates.MaxX-3), "+", GL_TRUE);
#endif
    GL_ComboBoxGrp_TypeDef* pTmp = (GL_ComboBoxGrp_TypeDef*)(pGUIObj->PrimaryComboOpt->objPTR);
    for (; index < pTmp->ComboOptionCount-1 && pTmp->ComboOptions[index]->Active == GL_FALSE; index++)
    {}
    GL_DrawFilledRectangle( minX - 1,
                            minY,
                            MAX_COMBO_LABEL_LENGTH*FONT_LENGTH,
                            COMBOBOX_SIZE,
                            GL_White);
    GL_DrawRectangle( minX - 1,
                      minY,
                      MAX_COMBO_LABEL_LENGTH*FONT_LENGTH,
                      COMBOBOX_SIZE);


    GL_SetFont(GL_FONT_SMALL);
    if ( pTmp->ComboOptionCount > 0)
    {
      GL_DisplayAdjStringLine( (uint8_t)(objCoordinates.MaxY - 16),
                               (uint16_t)(objCoordinates.MaxX) - (COMBOBOX_SIZE + 4),
                               (char*) pTmp->ComboOptions[index]->Label,
                               GL_TRUE);
    }
  }






  return GL_OK;
}















/**
  * @brief  Show the Control Object at the specified coordinates.
  * @param  *pThis: Pointer to Object Structure
  * @param  *objCoordinates: Pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetIconVisible(GL_PageControls_TypeDef* pTmp, GL_Coordinates_TypeDef objCoordinates)
{
  GL_Icon_TypeDef* pThis = (GL_Icon_TypeDef*)(pTmp->objPTR);
  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;
  /*GL_DrawColorBMP( (uint8_t*)pThis->ImagePTR,
                   (objCoordinates.MaxX),
                   (objCoordinates.MinY),
                   pThis->ImageWidth,
                   pThis->ImageHeight);*/
  GL_DrawButtonBMP( (uint16_t)(objCoordinates.MaxX),
                    (uint16_t)(objCoordinates.MinX),
                    (uint8_t)(objCoordinates.MaxY),
                    (uint8_t)(objCoordinates.MinY),
                    (uint8_t*)pThis->ImagePTR );

  return GL_OK;
}

/**
  * @brief  Show the Control Object at the specified coordinates.
  * @param  *pThis: Pointer to Object Structure
  * @param  *objCoordinates: Pointer to objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  pThis: pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetHistogramVisible (GL_PageControls_TypeDef* pTmp,
                                         GL_Coordinates_TypeDef objCoordinates )
{
  GL_Histogram_TypeDef* pGUIObj = (GL_Histogram_TypeDef*)(pTmp->objPTR);

  uint16_t barHeight;


  uint32_t index = 0;
  uint8_t x_step = 0;
  uint16_t height, width, max_x, offset_x, offset_y;
  double base_y, y_step, max_value = 0, y_resolution = 0;
  uint8_t value[7];


  if (!pGUIObj)
  {
    return GL_ERROR;
  }
  pGUIObj->Control_Visible = GL_TRUE;
  max_x = LCD_Width - HIST_MARGIN_LENGTH * 5 / 2;
  base_y = LCD_Height - 3 * HIST_MARGIN_LENGTH - 12;

  offset_x = (uint16_t)(max_x + 4);
  offset_y = (uint16_t)(base_y - 4);

  width  = LCD_Width  - 3 * HIST_MARGIN_LENGTH;
  height = LCD_Height - 4 * HIST_MARGIN_LENGTH;

  x_step = width / (pGUIObj->NrOfPoints);
  y_step = (height / 10);

  GL_SetForegroundColor(GL_Black);
  GL_SetTextColor(GL_Black);
  GL_SetFont(GL_FONT_SMALL);
  GL_DrawLine( max_x, (uint16_t)base_y, width, GL_Horizontal );        /* X Axis */
  GL_DrawLine( max_x, (uint16_t)(base_y - height), height, GL_Vertical );  /* Y Axis */
  GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), max_x + 5, "0", GL_TRUE );
  GL_DisplayAdjStringLine( offset_y - 5, offset_x + HIST_MARGIN_LENGTH - 9, "0", GL_TRUE );

  // Find maximum value of samples
  for (index = 0; index < (pGUIObj->NrOfPoints); index++)
  {
    if ( pGUIObj->Points[index] > max_value )
    {
      max_value = pGUIObj->Points[index];
    }
  }

  y_resolution = max_value / (height - y_step);

  /* Print the Y axis scale */
  for (index = 0; index < 10; index++)
  {
    GL_SetForegroundColor(GL_Black);
    GL_SetTextColor(GL_Black);
    GL_DrawLine( offset_x, (uint16_t)(offset_y - y_step*index), 4, GL_Horizontal );
    if ( index == 9)
    {
      GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 2 - FONT_LENGTH*2),
                               offset_x + FONT_LENGTH*5,
                               (char*) pGUIObj->Label_Y,
                               GL_TRUE );

      sprintf((char*)value, "%4.1f", max_value);

      if (max_value < 1000)
      {
        GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 5),
                                 offset_x + HIST_MARGIN_LENGTH*2,
                                 (char*) value,
                                 GL_TRUE );
      }
      else
      {
        GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 5),
                                 offset_x + FONT_LENGTH + HIST_MARGIN_LENGTH*2,
                                 (char*) value,
                                 GL_TRUE );
      }
    }
  }


  // Draw bars and print the X axis scale
  for (index = 0; index < (pGUIObj->NrOfPoints); index++)
  {
    barHeight = (uint16_t)(base_y - (double)((pGUIObj->Points[index]) / y_resolution));//?

    // Draw bar
    GL_DrawFilledRectangle( max_x - 1,//?
                            base_y - barHeight,
                            x_step - 1,//?
                            barHeight,//?
                            GL_Blue);

    max_x = max_x - x_step;

    GL_SetForegroundColor(GL_Black);
    GL_DrawLine( max_x + 1, (uint16_t)base_y, 4, GL_Vertical );//?

    if (pGUIObj->NrOfPoints <= 15)
    {
      sprintf((char*)value, "%d", (int)index + 1);
      GL_DisplayAdjStringLine( (uint16_t)base_y + 6, max_x + 6, (char*) value, GL_TRUE );
    }
    else if ( (pGUIObj->NrOfPoints) > 15 && (pGUIObj->NrOfPoints) <= 40)
    {
      if ((index + 1) % 3 == 0)
      {
        sprintf((char*)value, "%d", (int)index + 1);
        GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), max_x + 6, (char*) value, GL_TRUE );
      }
    }
    else
    {
      if ((index + 1) % 4 == 0)
      {
        sprintf((char*)value, "%d", (int)index + 1);
        GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), max_x + 6, (char*) value, GL_TRUE );
      }
    }
    if (index == (pGUIObj->NrOfPoints) - 1)
    {
      GL_DisplayAdjStringLine( (uint16_t)(base_y + 3 + FONT_LENGTH*2),//?
                               max_x + FONT_LENGTH*3,//?
                               (char*) (pGUIObj->Label_X),
                               GL_TRUE);
    }
  }
  return GL_OK;
}

/**
  * @brief  Show the Control Object at the specified coordinates.
  * @param  *pThis: Pointer to Object Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetGraphChartVisible( GL_PageControls_TypeDef* pTmp, GL_Coordinates_TypeDef objCoordinates )
{
  uint32_t index = 0;
  double height, width, offset_x, offset_y;
  __IO uint16_t signal_color = GL_Red;
  double base_y, y_step, x_step, max_value, y_resolution = 0, max_x;
  uint8_t value[7];
  GL_GraphChart_TypeDef* pThis = (GL_GraphChart_TypeDef*)(pTmp->objPTR);

  if (!pThis)
  {
    return GL_ERROR;
  }
  pThis->Control_Visible = GL_TRUE;

  max_value = 0;
  max_x = LCD_Width - GRAPH_MARGIN_LENGTH * 5 / 2 - FONT_LENGTH;
  base_y = LCD_Height - GRAPH_MARGIN_LENGTH * 7 / 2;

  offset_x = max_x + 4;
  offset_y = base_y;

  width  = LCD_Width  - GRAPH_MARGIN_LENGTH * 4;
  height = LCD_Height - GRAPH_MARGIN_LENGTH * 9 / 2;

  x_step = (double)(width) / (pThis->NrOfPoints);
  y_step = (double)(height / 11);

  GL_SetTextColor(GL_Black);
  GL_SetFont(GL_FONT_SMALL);
  GL_DrawLine( (uint16_t)max_x, (uint16_t)base_y, (uint16_t)(width + 2), GL_Horizontal );      /* X Axis */
  GL_DrawLine( (uint16_t)max_x, (uint16_t)(base_y - height), (uint16_t)height, GL_Vertical );  /* Y Axis */
  GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), (uint16_t)(max_x + 5), "0", GL_TRUE ); /* min X value*/
  GL_DisplayAdjStringLine( (uint16_t)(offset_y - 5), (uint16_t)(offset_x + GRAPH_MARGIN_LENGTH - 9), "0", GL_TRUE ); /* min Y value*/

  if (pThis->Background == GL_TRUE)
  {
    // Display the grid background color (black)
    GL_DrawFilledRectangle(max_x, base_y - height, width + 2, height, GL_Black);
    signal_color = GL_Yellow;
  }

  for (; index < (pThis->NrOfPoints); index++)
  {
    if ( pThis->Points[index] > max_value )
      max_value = pThis->Points[index];
  }

  y_resolution = (double)max_value / (height - y_step);

  /* Print the Y axis scale */
  for (index = 0; index <= 10; index++)
  {
    if (index > 0)
    {
      GL_SetForegroundColor(GL_Black);

      GL_DrawLine( (uint16_t)offset_x,
    		       (uint16_t)(offset_y - y_step*index),
                   4,
                   GL_Horizontal );

      GL_SetForegroundColor(GL_Grey);

      GL_DrawLine( (uint16_t)(offset_x - 4),
    		       (uint16_t)(offset_y - y_step*index),
                   (uint16_t)(width + 2),
                   GL_Horizontal );
    }
    /* Print the Y axis Label */
    if ( index == 10)
    {
      GL_SetTextColor(GL_Black);
      GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 2 - FONT_LENGTH*2),
                               (uint16_t)(offset_x + FONT_LENGTH*9 / 2),
                               (char*) (pThis->Label_Y),
                               GL_TRUE );

      sprintf((char*)value, "%4.1f", max_value);

      if (max_value < 1000)
      {
        GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 5),
                                 (uint16_t)(offset_x + GRAPH_MARGIN_LENGTH*2),
                                 (char*) value,
                                 GL_TRUE );
      }
      else
      {
        GL_DisplayAdjStringLine( (uint16_t)(offset_y - y_step*index - 5),
                                 (uint16_t)(offset_x + FONT_LENGTH + GRAPH_MARGIN_LENGTH*2),
                                 (char*) value,
                                 GL_TRUE );
      }
    }
  }

  /* Print the X axis scale */
  for (index = 0; index < (pThis->NrOfPoints); index++)
  {
    max_x = max_x - x_step;
    GL_SetForegroundColor(GL_Black);
    GL_DrawLine( (uint16_t)(max_x + 1), (uint16_t)base_y, 4, GL_Vertical );
    if (pThis->NrOfPoints <= 15)
    {
      sprintf((char*)value, "%d", (int)index + 1);
      GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), (uint16_t)(max_x + 6), (char*) value, GL_TRUE );
      GL_SetForegroundColor(GL_Grey);
      GL_DrawLine((uint16_t)(max_x + 1), (uint16_t)(base_y - height), (uint16_t)height, GL_Vertical );
    }
    else if ( pThis->NrOfPoints > 15 && pThis->NrOfPoints <= 40)
    {
      if ((index + 1) % 3 == 0)
      {
        sprintf((char*)value, "%d", (int)index + 1);
        GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), (uint16_t)(max_x + 6), (char*) value, GL_TRUE );
        GL_SetForegroundColor(GL_Grey);
        GL_DrawLine( (uint16_t)(max_x + 1), (uint16_t)(base_y - height), (uint16_t)height, GL_Vertical );
      }
    }
    else if ( pThis->NrOfPoints > 40 && pThis->NrOfPoints <= 70)
    {
      if ((index + 1) % 4 == 0)
      {
        sprintf((char*)value, "%d", (int)index + 1);
        GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), (uint16_t)(max_x + 6), (char*) value, GL_TRUE );
        GL_SetForegroundColor(GL_Grey);
        GL_DrawLine( (uint16_t)(max_x + 1), (uint16_t)(base_y - height), (uint16_t)height, GL_Vertical );
      }
    }
    else
    {
      if ((index + 1) % 10 == 0)
      {
        sprintf((char*)value, "%d", (int)index + 1);
        GL_DisplayAdjStringLine( (uint16_t)(base_y + 6), (uint16_t)(max_x + 6), (char*) value, GL_TRUE );
        GL_SetForegroundColor(GL_Grey);
        GL_DrawLine( (uint16_t)(max_x + 1), (uint16_t)(base_y - height), (uint16_t)height, GL_Vertical );
      }
    }
    /* Draw the Grid*/
    if (index == 0)
    {
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step),
                          (uint16_t)base_y,
                          (uint16_t)max_x,
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution)),
                          signal_color );
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step),
                          (uint16_t)(base_y - 1),
                          (uint16_t)max_x,
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution) - 1),
                          signal_color );
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step - 1),
                          (uint16_t)base_y,
                          (uint16_t)(max_x - 1),
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution)),
                          signal_color );
    }
    else
    {
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step),
                          (uint16_t)(base_y - (double)((pThis->Points[index-1]) / y_resolution)),
                          (uint16_t)max_x,
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution)),
                          signal_color );
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step),
                          (uint16_t)(base_y - (double)((pThis->Points[index-1]) / y_resolution) - 1),
                          (uint16_t)(max_x - 1),
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution) - 1),
                          signal_color );
      GL_DrawObliqueLine( (uint16_t)(max_x + x_step - 1),
                          (uint16_t)(base_y - (double)((pThis->Points[index-1]) / y_resolution)),
                          (uint16_t)(max_x - 1),
                          (uint16_t)(base_y - (double)((pThis->Points[index]) / y_resolution)),
                          signal_color );
    }
    /* Print the X axis Label */
    if (index == pThis->NrOfPoints - 1)
    {
      GL_SetTextColor(GL_Black);
      GL_DisplayAdjStringLine((uint16_t)(base_y + 3 + FONT_LENGTH*2),
                              (uint16_t)(max_x + FONT_LENGTH*3),
                              (char*) (pThis->Label_X),
                              GL_TRUE );
    }
  }
  GL_SetFont(GL_FONT_BIG);

  return GL_OK;
}

/**
  * @brief  Modify the points array of the GraphChart object
  * @param  *pPage: Pointer to Page Structure
  * @param  ID: Object Identifier
  * @param  data_points[]: The array of points to be plot on the LCD
  * @param  n_points: Number of points to be plot
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus SetGraphChartPoints( GL_Page_TypeDef* pPage, uint16_t ID, int16_t data_points[], uint8_t n_points )
{
  uint32_t index = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required object */
  {
    if (pPage->PageControls[index]->objType == GL_GRAPH_CHART)
    {
      GL_GraphChart_TypeDef* pTmp;
      pTmp = (GL_GraphChart_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        pTmp->NrOfPoints = n_points;

        /* Determine the length of the array of points and copy the content into the
         internal points array limiting the number of points to the maximum allowed */
        if ( n_points > MAX_GRAPH_POINTS )
        {
          memcpy(pTmp->Points, data_points, MAX_GRAPH_POINTS*2);
          pTmp->NrOfPoints = MAX_GRAPH_POINTS;
        }
        else
        {
          memcpy(pTmp->Points, data_points, n_points*2);
          pTmp->NrOfPoints = n_points;
        }

        return GL_OK;
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  Modify the points array of the Histogram object
  * @param  *pPage: Pointer to Page Structure
  * @param  ID: Object Identifier
  * @param  data_points[]: The array of points to be plot on the LCD
  * @param  n_points: Number of points to be plot
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus SetHistogramPoints( GL_Page_TypeDef* pPage, uint16_t ID, int16_t data_points[], uint8_t n_points )
{
  uint32_t index = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required object */
  {
    if (pPage->PageControls[index]->objType == GL_HISTOGRAM)
    {
      GL_Histogram_TypeDef* pTmp;
      pTmp = (GL_Histogram_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        pTmp->NrOfPoints = n_points;

        /* Determine the length of the array of points and copy the content into the
         internal points array limiting the number of points to the maximum allowed */
        if ( n_points > MAX_HIST_POINTS )
        {
          memcpy(pTmp->Points, data_points, MAX_HIST_POINTS*2);
          pTmp->NrOfPoints = MAX_HIST_POINTS;
        }
        else
        {
          memcpy(pTmp->Points, data_points, n_points*2);
          pTmp->NrOfPoints = n_points;
        }

        return GL_OK;
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  Shows the Control Object at the specified coordinates.
  * @param  *pObj:          pointer to Object Structure
  * @param  objCoordinates: objCoordinates Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise.
  */
static GL_ErrStatus SetTextboxVisible (GL_PageControls_TypeDef* pObj,
                                       GL_Coordinates_TypeDef objCoordinates)
{
  // Pobierz wskaznik na obiekt typu Textbox
  GL_Textbox_TypeDef* pGUIObj = (GL_Textbox_TypeDef*)(pObj->objPTR);

  uint16_t i;
  uint16_t text_length;
  uint16_t text_rows_max, text_cols_max;
  char temp_text[(LCD_Width / SMALL_FONT_WIDTH) + 1];

  // Pobierz wspolrzedne obiektu
  uint16_t maxX = objCoordinates.MaxX;
  uint16_t minY = objCoordinates.MinY;

  // Jezeli wskaznik jest rowny NULL
  if (!pGUIObj)
  {
    // Zwroc blad
    return GL_ERROR;
  }

  // Ustaw kolor z jakim beda rysowane kontury elementow graficznych
  GL_SetForegroundColor(GL_Black);

  // Wyczysc obszar pod obiektem typu Textbox
  GL_DrawFilledRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height, GL_White );

  // Rysuj ramke obiektu typu Textbox
  GL_DrawRectangle( maxX, minY, pGUIObj->Width, pGUIObj->Height);

  if (pGUIObj->Text != NULL)
  {
    // Wyznacz maksymalna liczbe znakow w wierszu i maksymalna liczbe wierszy
    text_cols_max = (pGUIObj->Width - 6) / SMALL_FONT_WIDTH;
    text_rows_max = (pGUIObj->Height - 4) / SMALL_FONT_HEIGHT;

    // Wyznacz liczbe znakow w tekscie
    text_length = strlen((char*)(pGUIObj->Text));

    GL_SetFont(GL_FONT_SMALL);
    GL_SetTextColor(GL_Black);

    // Rysuj znaki od lewej do prawej i od gory w dol
    if (text_length > text_cols_max)
    {
      i = 0;

      // Wykonuj dopoki sa wolne wiersze komponentu i oczekujace na wyswietlenie znaki
      while ( (text_length > i*text_cols_max) && (text_rows_max > i))
      {
        // Kopiuj podciag znakow do bufora tymczasowego
        strncpy(temp_text, ((char*)(pGUIObj->Text)) + i*text_cols_max, text_cols_max+1);

        // Jezeli ostatni wiersz komponentu
        if (i == text_rows_max - 1)
        {
          // Jezeli znakow jest wiecej niz mozna wyswietlic
          if ((temp_text[text_cols_max-1] != 0x0) && (text_length - i*text_cols_max) > text_cols_max)
          {
            // Ostatni znak w wierszu to ~
            temp_text[text_cols_max-1] = '~';
          }
        }

        // Zakoncz ciag znakiem NULL
        temp_text[text_cols_max] = 0x0;

        // Rysuj tekst
        GL_DrawString( maxX - 3, minY + 2 + i*SMALL_FONT_HEIGHT - (i != 0 ? 1 : 0), (uint8_t*)temp_text, GL_TRUE);

        ++i;
      }
    }
    else
    {
      // Rysuj tekst
      GL_DrawString( maxX - 3, minY + 2, pGUIObj->Text, GL_TRUE);
    }
  }
  return GL_OK;
}

/**
  * @brief  Gets a pointer to string of characters of the textbox object.
  * @param  *pPage: pointer to Page Structure.
  * @param  ID:     object ID.
  * @retval *pPageObj - the pointer to the string of characters.
  */
uint8_t* GetTextboxText (GL_Page_TypeDef* pPage,
                         uint16_t ID)
{
  GL_Textbox_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pObj = NULL;

  // Pobierz wskaznik do obiektu graficznego o zadanym ID na stronie (*pPage)
  pObj = GetObject(pPage, ID);

  // Jezeli obiekt nie istnieje
  if(pObj == NULL)
  {
    // Zwroc blad
    return NULL;
  }
  // W przeciwnym wypadku
  else
  {
    // Pobierz wskaznik na obiekt typu Textbox
    pGUIObj = (GL_Textbox_TypeDef*)pObj->objPTR;
    // Zwroc wskaznik na tekst komponentu
    return (uint8_t*)pGUIObj->Text;
  }
}

/**
  * @brief  Sets a new string of characters to the textbox object.
  * @param  *pPage: pointer to Page Structure.
  * @param  ID:     object ID.
  * @param  *text:  pointer to string of characters.
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise.
  */
GL_ErrStatus SetTextboxText (GL_Page_TypeDef* pPage,
                             uint16_t ID,
                             const char* text)
{
  GL_Textbox_TypeDef* pGUIObj = NULL;
  GL_PageControls_TypeDef* pObj = NULL;
  char* pStr = NULL;

  // Jezeli tekst zawiera co najmniej jeden znak
  if (strlen(text) > 0)
  {
    // Pobierz wskaznik do obiektu graficznego o zadanym ID na stronie (*pPage)
    pObj = GetObject(pPage, ID);

    // Jezeli obiekt nie istnieje
    if(pObj == NULL)
    {
      // Zwroc blad
      return GL_ERROR;
    }
    // W przeciwnym wypadku
    else
    {
      // Pobierz wskaznik na obiekt typu Textbox
      pGUIObj = (GL_Textbox_TypeDef*)pObj->objPTR;

      // Zwolnij miejsce w pamieci RAM zajmowane przez stary ciag znakow
      free(pGUIObj->Text);

      // Zarezerwuj wymagana ilosc miejsca w pamieci RAM pod nowy ciag znakow
      // i przekopiuj tam znaki
      pStr = malloc(strlen(text));
      strcpy(pStr,text);

      // Ustaw wskaznik na tekst
      pGUIObj->Text = (uint8_t*)pStr;

      return GL_OK;
    }
  }
  else
  {
    return GL_ERROR;
  }
}


/**
  * @brief  Adds an item to the listbox object.
  * @param  pObj: pointer to the listbox object.
  * @param  text: item name.
  * @retval None
  */
GL_ErrStatus AddListboxItem (GL_PageControls_TypeDef* pObj,
                             char* text)
{
  // Allocate memory space for new item
  ListboxItem_TypeDef* pItem = (ListboxItem_TypeDef*)malloc(sizeof(ListboxItem_TypeDef));
  // Get pointer to GUI object
  GL_Listbox_TypeDef* listbox = (GL_Listbox_TypeDef*)pObj->objPTR;

  uint16_t text_length_max;
  char* pStr;

  if ( (!listbox) || (!pItem) || (text == NULL))
  {
    return GL_ERROR;
  }

  // If listbox is not empty
  if(listbox->Item != NULL)
  {
    // Find last listbox item
    while(listbox->Item->NextItem != NULL)
    {
       listbox->Item = listbox->Item->NextItem;
    }
    // Save pointer to previous item
    pItem->PrevItem = listbox->Item;
    // Save pointer to next item
    listbox->Item->NextItem = pItem;
  }
  else
  {
    // No previous item
    pItem->PrevItem = NULL;
  }

  pItem->NextItem = NULL;

  // Add item to listbox
  listbox->Item = pItem;

  // Calculate number of maximum allowed characters in item text
  text_length_max = (listbox->Width - SCROLLBAR_THICKNESS - 6) / SMALL_FONT_WIDTH;

  // If text is too long
  if(strlen(text) > text_length_max)
  {
    pStr = (char*)malloc(text_length_max+1);
    strncpy(pStr,text,text_length_max);
    pStr[text_length_max-1] = '~';
    pStr[text_length_max] = 0x0;
  }
  else
  {
    pStr = (char*)malloc(strlen(text));
    strcpy(pStr,text);
  }

  listbox->Item->Text = (uint8_t*)pStr;
  listbox->Item->ItemSelected = GL_FALSE;

  (listbox->NrOfItems) += 1;

  return GL_OK;
}

/**
  * @brief  Removes all items from the listbox object.
  * @param  pObj: pointer to the listbox object.
  * @retval None
  */
GL_ErrStatus RemoveListboxAllItems (GL_PageControls_TypeDef* pObj)
{
  // Get pointer to GUI object
  GL_Listbox_TypeDef* listbox = (GL_Listbox_TypeDef*)pObj->objPTR;

  int16_t i;

  // Return error code if listbox is NULL or list is empty
  if ( !listbox || !(listbox->Item) )
  {
    return GL_ERROR;
  }

  // Find first item in list
  while (listbox->Item->PrevItem != NULL)
  {
    listbox->Item = listbox->Item->PrevItem;
  }

  for(i = 0; i < listbox->NrOfItems; i++)
  {
    if(listbox->Item->NextItem != NULL)
    {
      listbox->Item = listbox->Item->NextItem;
      free(listbox->Item->PrevItem);
      listbox->Item->PrevItem = NULL;
    }
    else
    {
      free(listbox->Item);
      listbox->Item = NULL;
    }
  }

  listbox->NrOfItems = 0;
  listbox->NrOfFirstItem = 0;

  return GL_OK;
}

/**
  * @brief  Removes selected items from the listbox object.
  * @param  pObj: pointer to the listbox object.
  * @retval None
  */
GL_ErrStatus RemoveListboxSelectedItems (GL_PageControls_TypeDef* pObj)
{
  // Get pointer to GUI object
  GL_Listbox_TypeDef* listbox = (GL_Listbox_TypeDef*)pObj->objPTR;
  // Define temporary pointers
  ListboxItem_TypeDef* pPrevItem;
  ListboxItem_TypeDef* pNextItem;

  int16_t i;

  // Return error code if listbox is NULL or list is empty
  if ( !listbox || !(listbox->Item) )
  {
    return GL_ERROR;
  }

  // Find first item in list
  while (listbox->Item->PrevItem != NULL)
  {
    listbox->Item = listbox->Item->PrevItem;
  }

  for(i = 0; i < listbox->NrOfItems; i++)
  {
    // If item is selected
    if(listbox->Item->ItemSelected)
    {
      // If first item in list and list has only 1 item
      if( (listbox->Item->PrevItem == NULL) && (listbox->Item->NextItem == NULL) )
      {
        free(listbox->Item);
        listbox->Item = NULL;
      }
      // If first item in list and list has more than 1 item
      else if( (listbox->Item->PrevItem == NULL) && (listbox->Item->NextItem != NULL) )
      {
        listbox->Item = listbox->Item->NextItem;
        free(listbox->Item->PrevItem);
        listbox->Item->PrevItem = NULL;
      }
      // If middle item in list
      else if( (listbox->Item->PrevItem != NULL) && (listbox->Item->NextItem != NULL) )
      {
        pPrevItem = listbox->Item->PrevItem;
        pNextItem = listbox->Item->NextItem;
        listbox->Item = listbox->Item->PrevItem;
        free(listbox->Item->NextItem);
        listbox->Item->NextItem = pNextItem;
        listbox->Item = listbox->Item->NextItem;
        listbox->Item->PrevItem = pPrevItem;
      }
      // If last item in list
      else//( (listbox->Item->PrevItem != NULL) && (listbox->Item->NextItem == NULL) )
      {
        listbox->Item = listbox->Item->PrevItem;
        free(listbox->Item->NextItem);
        listbox->Item->NextItem = NULL;
      }
      --i;
      --(listbox->NrOfItems);
    }
    else if(listbox->Item->NextItem != NULL)
    {
      listbox->Item = listbox->Item->NextItem;
    }
  }
  listbox->NrOfFirstItem = 0;
  return GL_OK;
}

/**
  * @brief  Updates state of the listbox object.
  * @param  *pObj:  pointer to the listbox object.
  * @param  Xtouch: x coordinate of touched point
  * @param  Ytouch: y coordinate of touched point
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus UpdateListboxState (GL_PageControls_TypeDef* pObj,
                                 uint32_t Xtouch,
                                 uint32_t Ytouch)
{
  // Get pointer to GUI object
  GL_Listbox_TypeDef* pGUIObj = (GL_Listbox_TypeDef*)(pObj->objPTR);

  uint16_t i, selectedItem, temp;

  uint16_t minX = pObj->objCoordinates.MinX;
  uint16_t maxY = pObj->objCoordinates.MaxY;
  uint16_t minY = pObj->objCoordinates.MinY;

  if (!pGUIObj)
  {
    return GL_ERROR;
  }

  // If item was selected
  if( Xtouch > (minX + 2 + SCROLLBAR_THICKNESS + 4) )
  {
    // Calculate which item was selected
    selectedItem = ( (Ytouch - (minY + 2)) / SMALL_FONT_HEIGHT ) + pGUIObj->NrOfFirstItem;

    // If no one item was selected (user has clicked on empty place)
    if(selectedItem >= pGUIObj->NrOfItems)
    {
      pGUIObj->NrOfLastSelItem = -1;
      return GL_OK;
    }

    pGUIObj->NrOfLastSelItem = selectedItem;

    // Find first item in list
    while (pGUIObj->Item->PrevItem != NULL)
    {
      pGUIObj->Item = pGUIObj->Item->PrevItem;
    }

    // If MultiSelection mode disabled
    if(pGUIObj->MultiSelection == GL_FALSE)
    {
      // Deselect all items
      for(i = 0; i < pGUIObj->NrOfItems; i++)
      {
        // Toggle state of the selected item
        if(i == selectedItem){
          if(pGUIObj->Item->ItemSelected == GL_TRUE){
            pGUIObj->Item->ItemSelected = GL_FALSE;
          }
          else{
            pGUIObj->Item->ItemSelected = GL_TRUE;
          }
        }
        // Deselect item
        else
        {
          pGUIObj->Item->ItemSelected = GL_FALSE;
        }

        // If next item exists
        if(pGUIObj->Item->NextItem != NULL)
        {
          // Change to next item
          pGUIObj->Item = pGUIObj->Item->NextItem;
        }
      }
    }
    // MultiSelection mode enabled
    else
    {
      // Find selected item in list
      for(i = 0; i < selectedItem; i++)
      {
        pGUIObj->Item = pGUIObj->Item->NextItem;
      }

      // Toggle state of the selected item
      if(pGUIObj->Item->ItemSelected == GL_TRUE){
        pGUIObj->Item->ItemSelected = GL_FALSE;
      }
      else{
        pGUIObj->Item->ItemSelected = GL_TRUE;
      }
    }
  }
  // Vertical Scrollbar selected
  else
  {
    // Upper arrow touched
    if( (Ytouch > minY) & (Ytouch < (minY + SCROLLBAR_ARROW_LENGTH + 4)) )
    {
      if(pGUIObj->NrOfFirstItem > 0)
      {
        (pGUIObj->NrOfFirstItem)--;
      }
    }
    // Lower arrow touched
    else if( (Ytouch > (maxY - SCROLLBAR_ARROW_LENGTH - 4)) & (Ytouch < maxY) )
    {
      // Calculate number of visible items
      temp = (uint16_t) ( (pGUIObj->Height - 2) / SMALL_FONT_HEIGHT );

      if( (temp + pGUIObj->NrOfFirstItem) < (pGUIObj->NrOfItems) )
      {
        (pGUIObj->NrOfFirstItem)++;
      }
    }
  }
  return GL_OK;
}

/**
  * @brief  Return the status of the object:
  *            - GL_FALSE: the object is not Hit
  *            - GL_TRUE: the object has been Hit
  *            - GL_NULL represent that no object are there with that ID
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @retval GL_bool - GL_FALSE, GL_TRUE
  */
GL_bool GetObjStatus(GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint32_t index = 0;

  if (pPage)
  {
    while ( index < pPage->ControlCount ) /* search for the required object */
    {
      if (pPage->PageControls[index]->objType == GL_BUTTON)
      {
        GL_Button_TypeDef* pTmp;
        pTmp = (GL_Button_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pTmp->ID == ID )
        {
          return pTmp->ObjectTouched;
        }
      }
      else if (pPage->PageControls[index]->objType == GL_SWITCH)
      {
        GL_Switch_TypeDef* pTmp;
        pTmp = (GL_Switch_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pTmp->ID == ID )
        {
          return pTmp->ObjectTouched;
        }
      }
      else if (pPage->PageControls[index]->objType == GL_CHECKBOX)
      {
        GL_Checkbox_TypeDef* pTmp;
        pTmp = (GL_Checkbox_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pTmp->ID == ID )
        {
          return pTmp->Checked;
        }
      }

      else if (pPage->PageControls[index]->objType == GL_ICON)
      {
        GL_Icon_TypeDef* pTmp;
        pTmp = (GL_Icon_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pTmp->ID == ID )
        {
          return pTmp->ObjectTouched;
        }
      }
      index++;
    }
  }
  return (GL_bool)GL_NULL;
}

void* GetObject (GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint16_t i = 0;

  if (!pPage)
  {
    return NULL;
  }

  // Search for the required object
  while ( i < pPage->ControlCount )
  {
    if ( pPage->PageControls[i]->ID == ID )
    {
      return (pPage->PageControls[i]);
    }
    i++;
  }

  // Object not found
  return NULL;
}

/**
  * @brief  Return the active combo option number
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @retval uint8_t - j: The active combo option index if the object was found
  *                 GL_NULL - if the object was not found
  */
uint8_t GetComboOptionActive(GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint32_t index = 0, combobox_index = 0;

  if (!pPage)
  {
    return GL_NULL;
  }

  while ( index < pPage->ControlCount ) /* search for the required combobox */
  {
    if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        while ( combobox_index < pTmp->ComboOptionCount )
        {
          if (pTmp->ComboOptions[combobox_index]->Active == GL_TRUE)
          {
            return combobox_index + 1;
          }
          combobox_index++;
        }
      }
    }
    index++;
  }
  return GL_NULL;
}

/**
  * @brief  Reset the active combo option to the first one
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus ResetComboOptionActive(GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint32_t index = 0, combobox_index = 0;

  if (pPage)
  {
    while ( index < pPage->ControlCount ) /* search for the required combobox */
    {
      if (pPage->PageControls[index]->objType == GL_COMBOBOX)
      {
        GL_ComboBoxGrp_TypeDef* pTmp;
        pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

        if ( pTmp->ID == ID )
        {
          while ( combobox_index < pTmp->ComboOptionCount )
          {
            if (combobox_index == 0)
            {
              pTmp->ComboOptions[combobox_index]->Active = GL_TRUE;
            }
            else
            {
              pTmp->ComboOptions[combobox_index]->Active = GL_FALSE;
            }
            combobox_index++;
          }
          return GL_OK;
        }
      }
      index++;
    }
  }
  return GL_ERROR;
}

/**
  * @brief  Return the active combo option number
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @retval uint8_t* - label: The active combo option label if the object was found
  *                 GL_NULL - if the object was not found
  */
const uint8_t* GetComboOptionLabel(GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint32_t index = 0, combobox_index = 0;
  if (!pPage)
  {
    return GL_NULL;
  }

  while ( index < pPage->ControlCount ) /* search for the required combobox */
  {
    if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
        while ( combobox_index < pTmp->ComboOptionCount )
        {
          if (pTmp->ComboOptions[combobox_index]->Active == GL_TRUE)
          {
            return pTmp->ComboOptions[combobox_index]->Label;
          }
          combobox_index++;
        }
    }
    index++;
  }
  return GL_NULL;
}

/**
  * @brief  Set the label for the active ComboOption
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @param  label: New label of the Active ComboOption Object
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus SetComboOptionLabel(GL_Page_TypeDef* pPage, uint16_t ID, const uint8_t* label)
{
  uint32_t index = 0, combobox_index = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required combobox */
  {
    if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        while ( combobox_index < pTmp->ComboOptionCount )
        {
          if (pTmp->ComboOptions[combobox_index]->Active == GL_TRUE)
          {
            if (p_strlen(label) > 0)
            {
              GL_SetStringFieldValue(pTmp->ComboOptions[combobox_index]->Label, (uint8_t *)label, MAX_COMBO_LABEL_LENGTH);

              return GL_OK;
            }
          }
          combobox_index++;
        }
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  Set the image pointer of the icon object
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @param  *pImage: Pointer to the image
  * @param  Width: Image Width
  * @param  Height: Image Height
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus SetIconImage(GL_Page_TypeDef* pPage, uint16_t ID, const uint8_t* pImage, uint16_t Width, uint8_t Height)
{
  uint32_t index = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }
  while ( index < pPage->ControlCount ) /* search for the required Icon object */
  {
    if (pPage->PageControls[index]->objType == GL_ICON)
    {
      GL_Icon_TypeDef* pTmp = NULL;

      pTmp = (GL_Icon_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        pTmp->ImagePTR = pImage;
        pTmp->ImageWidth = Width;
        pTmp->ImageHeight = Height;
        return GL_OK;
      }
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  This function return the object coordinates (X & Y)
  * @param  *pPage: pointer to the Page structure
  * @param  ID: Object Identifier
  * @retval GL_Coordinates_TypeDef - objCoordinates: The Object printing coordinate
  */
static GL_Coordinates_TypeDef GetObjCoordinates(GL_Page_TypeDef* pPage, uint16_t ID)
{
  GL_Coordinates_TypeDef null;
  uint32_t index = 0;

  /* Init the structure with zeros */
  memset(&null,0x00,sizeof(GL_Coordinates_TypeDef));

  if (!pPage)
  {
    return null;
  }

  while ( index < pPage->ControlCount ) /* search for the required object */
  {
    if (pPage->PageControls[index]->objType == GL_LABEL)
    {
      GL_Label_TypeDef* pTmp;
      pTmp = (GL_Label_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_BUTTON)
    {
      GL_Button_TypeDef* pTmp;
      pTmp = (GL_Button_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_BUTTON_COLOR)
    {
      GL_ButtonColor_TypeDef* pTmp;
      pTmp = (GL_ButtonColor_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_SWITCH)
    {
      GL_Switch_TypeDef* pTmp;
      pTmp = (GL_Switch_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_CHECKBOX)
    {
      GL_Checkbox_TypeDef* pTmp;
      pTmp = (GL_Checkbox_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_RADIO_BUTTON)
    {
      GL_RadioButtonGrp_TypeDef* pTmp;
      pTmp = (GL_RadioButtonGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_COMBOBOX)
    {
      GL_ComboBoxGrp_TypeDef* pTmp;
      pTmp = (GL_ComboBoxGrp_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_ICON)
    {
      GL_Icon_TypeDef* pTmp;
      pTmp = (GL_Icon_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
    if (pPage->PageControls[index]->objType == GL_SLIDEBAR)
    {
      GL_Slidebar_TypeDef* pTmp;
      pTmp = (GL_Slidebar_TypeDef*)(pPage->PageControls[index]->objPTR);

      if ( pTmp->ID == ID )
      {
        return pPage->PageControls[index]->objCoordinates;
      }
      index++;
    }
  }
  return null;
}

/**
  * @brief  This function return the object Lenght size (X & Y)
  * @param  *pPageControl: Pointer to PageControls object
  * @retval GL_ObjDimensions_TypeDef - dimensions: Control Object Lenght
  */
static GL_ObjDimensions_TypeDef GetObjSize(GL_PageControls_TypeDef* pPageControl)
{
  GL_ObjDimensions_TypeDef dimensions;

  memset(&dimensions, 0x00, sizeof(GL_ObjDimensions_TypeDef));

  if (pPageControl->objType == GL_BUTTON)
  {
    GL_Button_TypeDef* pTmp;
    pTmp = (GL_Button_TypeDef*)(pPageControl->objPTR);
    dimensions.Length = BUTTON_SLICE_LENGTH * 2 + (BUTTON_SLICE_LENGTH * (p_strlen((pTmp->Label)) - 1));
    dimensions.Height  = BUTTON_HEIGHT;
  }
  else if (pPageControl->objType == GL_BUTTON_COLOR)
  {
    dimensions.Length = 28;
    dimensions.Height = 28;
  }
  else if (pPageControl->objType == GL_SWITCH)
  {
    GL_Switch_TypeDef* pTmp;
    pTmp = (GL_Switch_TypeDef*)(pPageControl->objPTR);
    if ( p_strlen((pTmp->Label_1)) > p_strlen((pTmp->Label_2)) )
      dimensions.Length = BUTTON_SLICE_LENGTH * 2 + (BUTTON_SLICE_LENGTH * (p_strlen((pTmp->Label_1)) - 1));
    else
      dimensions.Length = BUTTON_SLICE_LENGTH * 2 + (BUTTON_SLICE_LENGTH * (p_strlen((pTmp->Label_2)) - 1));
    dimensions.Height  = BUTTON_HEIGHT;
  }
  else if (pPageControl->objType == GL_CHECKBOX)
  {
    dimensions.Length = CHECKBOX_SIZE;
    dimensions.Height = CHECKBOX_SIZE;
  }
  else if (pPageControl->objType == GL_RADIO_BUTTON)
  {
    dimensions.Length = RBUTTON_OPT_SIZE;
    dimensions.Height = RBUTTON_OPT_SIZE;
  }
  else if (pPageControl->objType == GL_COMBOBOX)
  {
    dimensions.Length = COMBOBOX_SIZE;
    dimensions.Height = COMBOBOX_SIZE;
  }
  else if (pPageControl->objType == GL_SLIDEBAR)
  {
    GL_Slidebar_TypeDef* pTmp;
    pTmp = (GL_Slidebar_TypeDef*)(pPageControl->objPTR);
    if (pTmp->Direction == GL_HORIZONTAL)
    {
      dimensions.Length = pTmp->Length;
      dimensions.Height = pTmp->Width;
    }
    else
    {
      dimensions.Height = pTmp->Length;
      dimensions.Length = pTmp->Width;
    }
  }
  else if (pPageControl->objType == GL_LISTBOX)
  {
    GL_Listbox_TypeDef* pTmp;
    pTmp = (GL_Listbox_TypeDef*)(pPageControl->objPTR);
    dimensions.Length = pTmp->Width;
    dimensions.Height = pTmp->Height;
  }
  else if (pPageControl->objType == GL_DRAWBOX)
  {
    GL_Drawbox_TypeDef* pTmp;
    pTmp = (GL_Drawbox_TypeDef*)(pPageControl->objPTR);
    dimensions.Length = pTmp->Width;
    dimensions.Height = pTmp->Height;
  }
  return dimensions;
}

/**
  * @brief  This function Enable or Disable (Hide) the Page.
  * @param  *pPage: Pointer to Page Structure
  * @param  bVal: GL_TRUE Enable / GL_FALSE Disable (Hide) the Page.
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
static GL_ErrStatus SetPage(GL_Page_TypeDef* pPage, GL_bool bVal)
{
  if (!pPage)
  {
    return GL_ERROR;
  }
  pPage->Page_Active = bVal;
  return GL_OK;
}

/**
  * @brief  This function Show or Hide the Page.
  * @param  *pPage: Pointer to Page Structure
  * @param  bVal: GL_TRUE Show / GL_FALSE Hide the Page.
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus ShowPage(GL_Page_TypeDef* pPage, GL_bool bVal)
{
  uint32_t i = 0;
  if (!pPage)
  {
    return GL_ERROR;
  }
  pPage->Page_Visible = bVal;
  pPage->SetPage(pPage, bVal);
  if (bVal == GL_TRUE)
  {
    while ( i < pPage->ControlCount ) /* search for the required button */
    {
      pPage->PageControls[i]->SetObjVisible( pPage->PageControls[i], pPage->PageControls[i]->objCoordinates);
      i++;
    }
    /*Read behind the cursor at specified position*/
    //CursorDraw(Cursor->X, Cursor->Y, CUR_READ_BEH);
  }
  else
  {
    GL_Clear(GL_White);
    //CursorDraw(Cursor->X, Cursor->Y, CUR_READ_BEH);
  }
  pPage->SetPage(pPage, bVal);

  return GL_OK;
}

/**
  * @brief  Refresh the Page if the status of any object was changed.
  * @param  *pPage: Pointer to Page Structure
  * @retval None
  */
void RefreshPage(GL_Page_TypeDef* pPage)
{
  ShowPage( pPage, GL_FALSE );
  ShowPage( pPage, GL_TRUE );
}

/**
  * @brief  Refresh the visualization of a Page Control Object.
  * @param  *pPage: Pointer to Page Structure
  * @param  *pPage: Pointer to Page Structure
  * @retval GL_ErrStatus - GL_OK if successful, GL_ERROR otherwise
  */
GL_ErrStatus RefreshPageControl( GL_Page_TypeDef* pPage, uint16_t ID)
{
  uint32_t index = 0;

  if (!pPage)
  {
    return GL_ERROR;
  }

  while ( index < pPage->ControlCount ) /* search for the required object */
  {
    if ( pPage->PageControls[index]->ID == ID )
    {
      pPage->PageControls[index]->SetObjVisible( pPage->PageControls[index], pPage->PageControls[index]->objCoordinates);
      return GL_OK;
    }
    index++;
  }
  return GL_ERROR;
}

/**
  * @brief  ChangePage the current Page showed on the LCD.
  * @param  *pPageOld: Pointer to Page Structure of the current page
  * @param  *pPageNew: Pointer to Page Structure of the new page
  * @retval None
  */
void ChangePage(GL_Page_TypeDef* pPageOld, GL_Page_TypeDef* pPageNew)
{
  ShowPage( pPageOld, GL_FALSE );
  ShowPage( pPageNew, GL_TRUE );
}

/**
  * @brief  Manage the the process of printing and changing behaviour of a touched
  *         control object
  * @param  *pControl: Pointer to PageControl object
  * @retval None
  */
static void CallPreEvents(GL_PageControls_TypeDef* pControl)
{
  uint32_t index = 0;
  uint32_t active_index = 0; /* needed for ComboBox */
#ifndef USE_2D_OBJECTS
  uint8_t* ptrBitmap = GL_NULL;
#endif
  GL_Coordinates_TypeDef tmpCoord;
  void* pTmp;
  GL_RadioButtonGrp_TypeDef* pTmpGrp = GL_NULL;
  GL_ComboBoxGrp_TypeDef* pTmpComboGrp = GL_NULL;

  switch (pControl->objType)
  {
    case GL_BUTTON:
      pTmp = (GL_Button_TypeDef*)(pControl->objPTR);
      ((GL_Button_TypeDef*)(pTmp))->ObjectTouched = GL_TRUE;
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      while (index < 100000)
        index++;
      ((GL_Button_TypeDef*)(pTmp))->ObjectTouched = GL_FALSE;
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      break;
    case GL_CHECKBOX:
      pTmp = (GL_Checkbox_TypeDef*)(pControl->objPTR);
      if ( ((GL_Checkbox_TypeDef*)(pTmp))->Checked == GL_TRUE )
        ((GL_Checkbox_TypeDef*)(pTmp))->Checked = GL_FALSE;
      else
        ((GL_Checkbox_TypeDef*)(pTmp))->Checked = GL_TRUE;
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      break;
    case GL_SWITCH:
      pTmp = (GL_Switch_TypeDef*)(pControl->objPTR);
      if ( ((GL_Switch_TypeDef*)(pTmp))->ObjectTouched == GL_TRUE )
        ((GL_Switch_TypeDef*)(pTmp))->ObjectTouched = GL_FALSE;
      else
        ((GL_Switch_TypeDef*)(pTmp))->ObjectTouched = GL_TRUE;
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      break;
    case GL_RADIO_BUTTON:
      pTmpGrp = ((GL_RadioOption_TypeDef*)(pControl->objPTR))->RadioButtonGrp;
      pTmp = (GL_RadioOption_TypeDef*)(pControl->objPTR);
      if ( ((GL_RadioOption_TypeDef*)(pTmp))->Checked == GL_FALSE )
      {
        for (index = 0; index < pTmpGrp->RadioOptionCount; index++)
        {
          ( (GL_RadioOption_TypeDef*)(pTmpGrp->RadioOptions[index]->objPTR))->Checked = GL_FALSE;
        }
        ((GL_RadioOption_TypeDef*)(pTmp))->Checked = GL_TRUE;
        for (index = 0; index < (pTmpGrp->RadioOptionCount); index++)
        {
          pControl = pTmpGrp->RadioOptions[index];
          pControl->SetObjVisible(pControl, pControl->objCoordinates);
        }
      }
      break;
    case GL_COMBOBOX:
      pTmpComboGrp = (GL_ComboBoxGrp_TypeDef*)(pControl->objPTR);
#ifndef USE_2D_OBJECTS
      ptrBitmap = pTmpComboGrp->ImageClickedPTR;
      GL_DrawColorBMP( ptrBitmap,
                       pControl->objCoordinates.MaxX,
                       pControl->objCoordinates.MaxY,
                       COMBOBOX_SIZE,
                       COMBOBOX_SIZE);
      GL_Delay(20);
#else
      GL_DrawFilledRectangle( pControl->objCoordinates.MaxX,
                          pControl->objCoordinates.MinY,
                          COMBOBOX_SIZE,
                          COMBOBOX_SIZE,
                          GL_Cyan);
      GL_SetFont(GL_FONT_BIG);
#endif

      if (((GL_ComboBoxGrp_TypeDef*)(pTmpComboGrp))->Secondary == GL_FALSE)
      {
#ifdef USE_2D_OBJECTS
        GL_SetTextColor(GL_Grey);
        GL_DisplayAdjStringLine( (uint8_t)(pControl->objCoordinates.MinY-1), (uint16_t)(pControl->objCoordinates.MaxX-1), "-", GL_TRUE);
        GL_DisplayAdjStringLine( (uint8_t)(pControl->objCoordinates.MinY-1), (uint16_t)(pControl->objCoordinates.MaxX-4), "-", GL_TRUE);
        GL_Delay(20);
#endif
        for (index = 0; index < ((GL_ComboBoxGrp_TypeDef*)(pTmpComboGrp))->ComboOptionCount; index++)
        {
          if ( ((GL_ComboBoxGrp_TypeDef*)(pTmpComboGrp))->ComboOptions[index]->Active == GL_TRUE )
          {
            active_index = index;
          }
          ((GL_ComboBoxGrp_TypeDef*)(pTmpComboGrp))->ComboOptions[index]->Active = GL_FALSE;
        }
        if (active_index == 0)
          pTmpComboGrp->ComboOptions[active_index]->Active = GL_TRUE;
        if (active_index > 0)
          pTmpComboGrp->ComboOptions[active_index-1]->Active = GL_TRUE;
        pControl->SetObjVisible(pControl, pControl->objCoordinates);
      }
      else
      {
#ifdef USE_2D_OBJECTS
        GL_SetTextColor(GL_Grey);
        GL_DisplayAdjStringLine( (uint8_t)(pControl->objCoordinates.MinY),
                                 (uint16_t)(pControl->objCoordinates.MaxX-3),
                                 "+",
                                 GL_TRUE);
        GL_Delay(20);
#endif
        GL_ComboBoxGrp_TypeDef* pTmpComboGrp2 = ((GL_ComboBoxGrp_TypeDef*)(pControl->objPTR))->PrimaryComboOpt->objPTR;
        for (index = 0; index < pTmpComboGrp2->ComboOptionCount; index++)
        {
          if ( pTmpComboGrp2->ComboOptions[index]->Active == GL_TRUE )
          {
            active_index = index;
          }
          pTmpComboGrp2->ComboOptions[index]->Active = GL_FALSE;
        }

        if ( active_index == pTmpComboGrp2->ComboOptionCount - 1 )
          pTmpComboGrp2->ComboOptions[active_index]->Active = GL_TRUE;
        else if ( active_index < pTmpComboGrp2->ComboOptionCount - 1 )
          pTmpComboGrp2->ComboOptions[active_index+1]->Active = GL_TRUE;

        pControl->SetObjVisible(pControl, pControl->objCoordinates);
      }
      GL_SetFont(GL_FONT_SMALL);
      break;

    case GL_SLIDEBAR:
      pTmp = (GL_Slidebar_TypeDef*)(pControl->objPTR);

      /* GL_ObjDimensions_TypeDef tmpSize; */
      tmpCoord = pControl->objCoordinates;
      if ( ((GL_Slidebar_TypeDef*)pTmp)->Direction == GL_HORIZONTAL )
      {
        /* Click on the left of the cursor - decreasing the value */
        if ( u32_TSXCoordinate >= (tmpCoord.MaxX - ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue > 0 )
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - 5;
        }

        /* Click on the right of the cursor - increasing value */
        else if ( u32_TSXCoordinate <= (tmpCoord.MinX - ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - SLIDEBAR_CURSOR_LENGTH - SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue < 100)
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue + 5;
        }
      }
      else if ( ((GL_Slidebar_TypeDef*)pTmp)->Direction == GL_LEFT_VERTICAL )
      {
        /* Click on the LEFT of the cursor - decreasing the value */
        if ( u32_TSYCoordinate > (tmpCoord.MaxY - ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue > 0 )
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - 5;
        }

        /* Click on the RIGHT of the cursor - increasing value */
        else if ( u32_TSYCoordinate < (tmpCoord.MaxY - ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - SLIDEBAR_CURSOR_LENGTH*2 - SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue < 100)
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue + 5;
        }
      }
      else if ( ((GL_Slidebar_TypeDef*)pTmp)->Direction == GL_RIGHT_VERTICAL )
      {
        /* Click on the RIGHT of the cursor - decreasing the value */
        if ( u32_TSYCoordinate >= (tmpCoord.MinY + ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue + SLIDEBAR_CURSOR_LENGTH + SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue < 100 )
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue + 5;
        }

        /* Click on the LEFT of the cursor - increasing value */
        else if ( u32_TSYCoordinate <= (tmpCoord.MinY + ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - SLIDEBAR_OFFSET_LENGTH) && ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue > 0)
        {
          /* Calculate a new cursor position from new value */
          ((GL_Slidebar_TypeDef*)pTmp)->PreviousValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue;
          ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue = ((GL_Slidebar_TypeDef*)pTmp)->CurrentValue - 5;
        }
      }
      else
        break;
      CursorDraw(Cursor->X, Cursor->Y, CUR_READ_BEH);
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      break;
    case GL_LISTBOX:
      UpdateListboxState(pControl, u32_TSXCoordinate, u32_TSYCoordinate);
      pTmp = (GL_Listbox_TypeDef*)(pControl->objPTR);
      pControl->SetObjVisible(pControl, pControl->objCoordinates);
      break;
    case GL_DRAWBOX:
      Draw_Point(pControl, u32_TSXCoordinate, u32_TSYCoordinate);
      break;
    default:
      break;
  }
}

/**
  * @brief  Depending on the Coordinates of the Touch Screen pressed, this
  *         function does the processing and executes the proper function.
  * @param  None
  * @retval None
  */
void ProcessInputData(void)
{
  uint16_t p_index, c_index;
  GL_Coordinates_TypeDef tmpCoord;
  GL_ObjDimensions_TypeDef tmpSize;

  if ( touch_done)
  {
    if (vu8_gSleepState == 1)
    {
      GL_BackLightSwitch(GL_ON);
      vu8_gSleepState = 0;
    }
    else if ((vu8_gTouchEnable == 1) && (vu8_gSleepState == 0))
    {
      vu32_gTimeOutCount = 0;
      for ( p_index = 0; p_index < PageCount; p_index++)
      {
        if ( PagesList[p_index]->Page_Active == GL_TRUE )
        {
          for ( c_index = 0; c_index < PagesList[p_index]->ControlCount; c_index++ )
          {
        	  tmpCoord = PagesList[p_index]->PageControls[c_index]->objCoordinates;
            tmpSize = GetObjSize(PagesList[p_index]->PageControls[c_index]);
            if (touch_done)
            {
              if ( CompareCoordinates( tmpCoord.MaxX, tmpCoord.MaxX - tmpSize.Length + 1, tmpCoord.MaxY, tmpCoord.MaxY - tmpSize.Height ) )
              {
                CallPreEvents(PagesList[p_index]->PageControls[c_index]);
                CallEvent(PagesList[p_index]->PageControls[c_index]);
                u32_TSYCoordinate = 0;
                touch_done = 0;
                //GL_Delay(5);
                break;
              }
            }
          }
        }
      }
    }
    touch_done = 0;
  }
}

/**
  * @brief  "Do nothing" function
  * @param  None
  * @retval None
  */
void NullFunc(void)
{}

/**
  * @brief  Run the proper function associated to the touch event of a specific
  *         control object
  * @param  *pControl: pointer to PageControl object
  * @retval None
  */
static void CallEvent(GL_PageControls_TypeDef* pControl)
{
  uint32_t index = 0;
  void* pTmp;

  switch (pControl->objType)
  {
    case GL_BUTTON:
      pTmp = (GL_Button_TypeDef*)(pControl->objPTR);
      ((GL_Button_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_BUTTON_COLOR:
      pTmp = (GL_ButtonColor_TypeDef*)(pControl->objPTR);
      ((GL_ButtonColor_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_CHECKBOX:
      pTmp = (GL_Checkbox_TypeDef*)(pControl->objPTR);
      ((GL_Checkbox_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_SWITCH:
      pTmp = (GL_Switch_TypeDef*)(pControl->objPTR);
      ((GL_Switch_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_ICON:
      pTmp = (GL_Icon_TypeDef*)(pControl->objPTR);
      ((GL_Icon_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_RADIO_BUTTON:
      pTmp = (GL_RadioOption_TypeDef*)(pControl->objPTR);
      ((GL_RadioOption_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_COMBOBOX:
      pTmp = pControl->objPTR;
      if (((GL_ComboBoxGrp_TypeDef*)(pTmp))->Secondary == GL_FALSE )
      {
        for (; index < ((GL_ComboBoxGrp_TypeDef*)(pTmp))->ComboOptionCount-1 && ((GL_ComboBoxGrp_TypeDef*)(pTmp))->ComboOptions[index]->Active == GL_FALSE; index++)
        {}
        ((GL_ComboBoxGrp_TypeDef*)(pTmp))->ComboOptions[index]->EventHandler();
      }
      else
      {
        GL_ComboBoxGrp_TypeDef* pPrimaryOption = ((GL_ComboBoxGrp_TypeDef*)(((GL_ComboBoxGrp_TypeDef*)(pTmp))->PrimaryComboOpt->objPTR));
        for (; index < pPrimaryOption->ComboOptionCount-1 && pPrimaryOption->ComboOptions[index]->Active == GL_FALSE; index++)
        {}
        pPrimaryOption->ComboOptions[index]->EventHandler();
      }
      break;
    case GL_SLIDEBAR:
      pTmp = (GL_Slidebar_TypeDef*)(pControl->objPTR);
      ((GL_Slidebar_TypeDef*)pTmp)->EventHandler();
      break;
    case GL_LISTBOX:
      pTmp = (GL_Listbox_TypeDef*)(pControl->objPTR);
      if( ((GL_Listbox_TypeDef*)pTmp)->NrOfLastSelItem >= 0)
      {
        ((GL_Listbox_TypeDef*)pTmp)->EventHandler(((GL_Listbox_TypeDef*)pTmp)->NrOfLastSelItem);
      }
      break;
    default:
      break;
  }
}

/**
  * @brief  Set LCD Panel Resolution
  * @param  Lcd_Width: Width of the LCD Panel
  * @param  Lcd_Height: Height of the LCD Panel
  * @retval None
  */
void Set_LCD_Resolution( uint16_t Lcd_Width, uint16_t Lcd_Height )
{
  LCD_Width  = Lcd_Width;
  LCD_Height = Lcd_Height;
}

/**
  * @brief  This function checks the coordinates of the touch screen pressed
  *         converted in LCD ones and depending on the level of the menu
  *         checks if the press is valid or not.
  * @param  u16_XMax: Maximum X coordinate for valid touch
  * @param  u16_XMin: Minimum X coordinate for valid touch
  * @param  u8_YMax: Maximum Y coordinate for valid touch
  * @param  u8_YMin: Minimum Y coordinate for valid touch
  * @retval valid touch detected (1) or not-valid touch detected (0).
  */
uint8_t CompareCoordinates(uint16_t u16_XMax, uint16_t u16_XMin, uint16_t u16_YMax, uint16_t u16_YMin)
{
  if (( u32_TSXCoordinate <= u16_XMin) || ( u32_TSXCoordinate >= u16_XMax - 5))
  {
    ValidTouchDetected = 0;
    return 0;
  }
  else if (( u32_TSYCoordinate < u16_YMin) || ( u32_TSYCoordinate > u16_YMax))
  {
    ValidTouchDetected = 0;
    return 0;
  }
  else
  {
    ValidTouchDetected = 1;
    return 1;
  }
}

/**
  * @brief  This function checks the coordinates of the Joystick Pointer
  *         with the graphics object ones printed on the LCD and depending on
  *         the level of the menu checks if the press is valid or not.
  * @param  u16_XMax: Maximum X coordinate for valid press
  * @param  u16_XMin: Minimum X coordinate for valid press
  * @param  u8_YMax: Maximum Y coordinate for valid press
  * @param  u8_YMin: Minimum Y coordinate for valid press
  * @retval valid press detected (1) or not-valid press detected (0).
  */
uint8_t CompareJoyCoordinates(uint16_t u16_XMax, uint16_t u16_XMin, uint16_t u16_YMax, uint16_t u16_YMin)
{
  if ( ( u32_TSXCoordinate < u16_XMin) || (u32_TSXCoordinate > u16_XMax) )
  {
    ValidTouchDetected = 0;
    return 0;
  }
  else if ( ( u32_TSYCoordinate < u16_YMin) || ( u32_TSYCoordinate > u16_YMax) )
  {
    ValidTouchDetected = 0;
    return 0;
  }
  else
  {
    ValidTouchDetected = 1;
    return 1;
  }
}

/**
  * @brief  Switches the backlightOFF when power save mode is
  *         enabled and no event is detected for long time.
  * @param  None
  * @retval None
  */
void TimeOutCalculate(void)
{
  if (vu8_gSleepState == 0)
  {
    if (vu8_gPowerSaveOption == 1)
    {
      vu32_gTimeOutCount++;
      if (vu32_gTimeOutCount > TIMEOUT)
      {
        GL_BackLightSwitch(GL_OFF);
        vu8_gSleepState = 1;
        vu32_gTimeOutCount = 0;
      }
    }
  }
}

/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length (time base 10 ms).
  * @retval None
  */
void GL_Delay(uint32_t nTime)
{
  TimingDelay = nTime;

  while (TimingDelay != 0);

}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  {
    TimingDelay--;
  }
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/

